@@ -20,8 +20,10 @@ using System.Net;
 using System.Net.Sockets;
 using System.IO;
 using ICSharpCode.SharpZipLib.Zip.Compression;
+using ICSharpCode.SharpZipLib.Zip.Compression.Streams;
 using System.Security.Cryptography;
 using ByteFX.Data.Common;
+using System.Collections;
 using System.Text;
 
 namespace ByteFX.Data.MySqlClient
@@ -33,21 +35,23 @@ namespace ByteFX.Data.MySqlClient
 	{
 		protected const int HEADER_LEN = 4;
 		protected const int MIN_COMPRESS_LENGTH = 50;
+		protected const int MAX_PACKET_SIZE = 256*256*256-1;
 
-		protected MySqlStream		stream;
+		protected Stream			stream;
+		protected BufferedStream	writer;
 		protected Encoding			encoding;
 		protected byte				packetSeq;
 		protected long				maxPacketSize;
-		protected Packet			peekedPacket = null;
-		protected ByteFX.Data.Common.Version	serverVersion;
+		protected DBVersion			serverVersion;
 		protected bool				isOpen;
 		protected string			versionString;
+		protected Packet			peekedPacket;
 
-		int		protocol;
-		uint	threadID;
-		String	encryptionSeed;
-		int		serverCaps;
-		bool	useCompression = false;
+		protected int				protocol;
+		protected uint				threadID;
+		protected String			encryptionSeed;
+		protected int				serverCaps;
+		protected bool				useCompression = false;
 
 
 		public Driver()
@@ -69,21 +73,38 @@ namespace ByteFX.Data.MySqlClient
 			set { maxPacketSize = value; }
 		}
 
-		public string Version 
+		public string VersionString 
 		{
 			get { return versionString; }
 		}
 
+		public DBVersion Version 
+		{
+			get { return serverVersion; }
+		}
+
 		public void Open( MySqlConnectionString settings )
 		{
-			stream = new MySqlStream( settings.Host, settings.Port, 30, settings.ConnectTimeout );
+			// connect to one of our specified hosts
+			try 
+			{
+				StreamCreator sc = new StreamCreator( settings.Server, settings.Port, settings.PipeName );
+				stream = sc.GetStream( settings.ConnectionTimeout );
+			}
+			catch (Exception ex)
+			{
+				throw new MySqlException("Unable to connect to any of the specified MySQL hosts", ex);
+			}
 
+			if (stream == null) 
+				throw new MySqlException("Unable to connect to any of the specified MySQL hosts");
+
+			writer = new BufferedStream( stream );
+			// read off the welcome packet and parse out it's values
 			Packet packet = ReadPacket();
-
-			// read off the protocol version
 			protocol = packet.ReadByte();
 			versionString = packet.ReadString();
-			serverVersion = ByteFX.Data.Common.Version.Parse( versionString );
+			serverVersion = DBVersion.Parse( versionString );
 			threadID = (uint)packet.ReadInteger(4);
 			encryptionSeed = packet.ReadString();
 
@@ -92,10 +113,26 @@ namespace ByteFX.Data.MySqlClient
 			if (packet.HasMoreData)
 				serverCaps = (int)packet.ReadInteger(2);
 
-			Authenticate( settings.Username, settings.Password, settings.UseCompression );
+			Authenticate( settings.UserId, settings.Password, settings.UseCompression );
+
+			// if we are using compression, then we use our CompressedStream class
+			// to hide the ugliness of managing the compression
+			if (settings.UseCompression)
+			{
+				stream = new CompressedStream( stream );
+				writer = new BufferedStream( stream );
+			}
+
 			isOpen = true;
 		}
 
+		private Packet CreatePacket( byte[] buf )
+		{
+			if (buf == null)
+				return new Packet( serverVersion.isAtLeast(3, 22, 5) );
+			return new Packet( buf, serverVersion.isAtLeast(3, 22, 5 ));
+		}
+
 		private void Authenticate( String userid, String password, bool UseCompression )
 		{
 			ClientParam clientParam = ClientParam.CLIENT_FOUND_ROWS | ClientParam.CLIENT_LONG_FLAG;
@@ -109,12 +146,12 @@ namespace ByteFX.Data.MySqlClient
 			clientParam |= ClientParam.CLIENT_LOCAL_FILES;
 //			if (serverVersion.isAtLeast(4,1,0))
 //				clientParam |= ClientParam.CLIENT_PROTOCOL_41;
-			if ( (serverCaps & (int)ClientParam.CLIENT_SECURE_CONNECTION ) != 0 && password.Length > 0 )
-				clientParam |= ClientParam.CLIENT_SECURE_CONNECTION;
+//			if ( (serverCaps & (int)ClientParam.CLIENT_SECURE_CONNECTION ) != 0 && password.Length > 0 )
+//				clientParam |= ClientParam.CLIENT_SECURE_CONNECTION;
 
 			int packetLength = userid.Length + 16 + 6 + 4;  // Passwords can be 16 chars long
 
-			Packet packet = new Packet();// packetLength );
+			Packet packet = CreatePacket(null);
 
 			if ((clientParam & ClientParam.CLIENT_PROTOCOL_41) != 0)
 			{
@@ -137,6 +174,9 @@ namespace ByteFX.Data.MySqlClient
 			{
 				// use old authentication system
 				packet.WriteString( EncryptPassword(password, encryptionSeed, protocol > 9), encoding );
+				// pad zeros out to packetLength for auth
+				for (int i=0; i < (packetLength-packet.Length); i++)
+					packet.WriteByte(0);
 				SendPacket(packet);
 			}
 
@@ -175,64 +215,11 @@ namespace ByteFX.Data.MySqlClient
 			Security.ArrayCrypt( cryptSalt, 0, firstPassBytes, 0, firstPassBytes, 20 );
 
 			// send the packet
-			packet = new Packet();
+			packet = CreatePacket(null);
 			packet.Write( firstPassBytes, 0, 20 );
 			SendPacket(packet);
 		}
 
-		/// <summary>
-		/// 
-		/// </summary>
-		private Packet ReadRawPacket()
-		{
-			int packetLength = stream.ReadInt24();
-			int unCompressedLen = 0;
-
-			// read the packet sequence and make sure it makes sense
-			byte seq = (byte)stream.ReadByte();
-			if (seq != packetSeq) 
-				throw new MySqlException("Unknown transmission status: sequence out of order");
-	
-			if (useCompression) 
-				unCompressedLen = stream.ReadInt24();
-
-			byte[] buffer;
-			if (useCompression && unCompressedLen > 0)
-			{
-				byte[] compressed_buffer = new Byte[packetLength];
-				buffer = new Byte[unCompressedLen];
-
-				// read in the compressed data
-				stream.Read( compressed_buffer, 0, packetLength );
-
-				// inflate it
-				Inflater i = new Inflater();
-				i.SetInput( compressed_buffer );
-				i.Inflate( buffer );
-			}
-			else 
-			{
-				buffer = new Byte[packetLength];
-				stream.Read( buffer, 0, packetLength);
-			}
-
-			packetSeq++;
-			Packet packet = new Packet( buffer );
-			packet.Encoding = encoding;
-			return packet;
-		}
-
-		/// <summary>
-		/// 
-		/// </summary>
-		public void SendFileToServer()
-		{
-		}
-
-		public void ClearPeekedPacket()
-		{
-			peekedPacket = null;
-		}
 
 		/// <summary>
 		/// 
@@ -240,7 +227,6 @@ namespace ByteFX.Data.MySqlClient
 		/// <returns></returns>
 		public Packet PeekPacket()
 		{
-			// we can peek the same packet more than once
 			if (peekedPacket != null)
 				return peekedPacket;
 
@@ -249,116 +235,190 @@ namespace ByteFX.Data.MySqlClient
 		}
 
 		/// <summary>
-		/// 
+		/// ReadBuffer continuously loops until it has read the entire
+		/// requested data
+		/// </summary>
+		/// <param name="buf">Buffer to read data into</param>
+		/// <param name="offset">Offset to place the data</param>
+		/// <param name="length">Number of bytes to read</param>
+		private void ReadBuffer( byte[] buf, int offset, int length )
+		{
+			while (length > 0)
+			{
+				int amountRead = stream.Read( buf, offset, length );
+				if (amountRead == 0)
+					throw new MySqlException("Unexpected end of data encountered");
+				length -= amountRead;
+				offset += amountRead;
+			}
+		}
+
+		private Packet ReadPacketFromServer()
+		{
+			int len = stream.ReadByte() + (stream.ReadByte() << 8) +
+				(stream.ReadByte() << 16);
+			byte seq = (byte)stream.ReadByte();
+			byte[] buf = new byte[ len ];
+			ReadBuffer( buf, 0, len );
+
+			if (seq != packetSeq) 
+				throw new MySqlException("Unknown transmission status: sequence out of order");
+			packetSeq++;
+
+			Packet p = CreatePacket(buf);
+			p.Encoding = this.Encoding;
+			if (p.Length == MAX_PACKET_SIZE && serverVersion.isAtLeast(4,0,0)) 
+				p.Append( ReadPacketFromServer() );
+			return p;
+		}
+
+		/// <summary>
+		/// Reads a single packet off the stream
 		/// </summary>
 		/// <returns></returns>
 		public Packet ReadPacket()
 		{
-			// if we have a peeked packet, return it now
-			if (peekedPacket != null) 
+			// if we have peeked at a packet, then return it
+			if (peekedPacket != null)
 			{
-				Packet p = peekedPacket;
+				Packet packet = peekedPacket;
 				peekedPacket = null;
-				return p;
+				return packet;
 			}
 
-			Packet packet = ReadRawPacket();
+			Packet p = ReadPacketFromServer();
 
 			// if this is an error packet, then throw the exception
-			if (packet[0] == 0xff)
+			if (p[0] == 0xff)
 			{
-				packet.ReadByte();
-				int errorCode = (int)packet.ReadInteger(2);
-				string msg = packet.ReadString();
+				p.ReadByte();
+				int errorCode = (int)p.ReadInteger(2);
+				string msg = p.ReadString();
 				throw new MySqlException( msg, errorCode );
 			}
+			
+			return p;
+		}
 
-			return packet;
+		protected MemoryStream CompressBuffer(byte[] buf, int index, int length)
+		{
+
+			if (length < MIN_COMPRESS_LENGTH) return null;
+
+			MemoryStream ms = new MemoryStream(buf.Length);
+			DeflaterOutputStream dos = new DeflaterOutputStream(ms);
+
+			dos.WriteByte( (byte)(length & 0xff ));
+			dos.WriteByte( (byte)((length >> 8) & 0xff ));
+			dos.WriteByte( (byte)((length >> 16) & 0xff ));
+			dos.WriteByte( 0 );
+
+			dos.Write( buf, index, length );
+			dos.Finish();
+			if (ms.Length > length+4) return null;
+			return ms;
+		}
+
+		private void WriteInteger( int v, int numbytes )
+		{
+			int val = v;
+
+			if (numbytes < 1 || numbytes > 4) 
+				throw new ArgumentOutOfRangeException("Wrong byte count for WriteInteger");
+
+			for (int x=0; x < numbytes; x++)
+			{
+				writer.WriteByte( (byte)(val&0xff) );
+				val >>= 8;
+			}
 		}
 
 		/// <summary>
-		/// 
+		/// Send a buffer to the server in a compressed form
 		/// </summary>
-		/// <returns></returns>
-/*		protected byte[] CompressPacket(Packet packet)
+		/// <param name="buf">Byte buffer to send</param>
+		/// <param name="index">Location in buffer to start sending</param>
+		/// <param name="length">Amount of data to send</param>
+		protected void SendCompressedBuffer(byte[] buf, int index, int length)
 		{
-			// compress the entire packet except the length
+			MemoryStream compressed_bytes = CompressBuffer(buf, index, length);
+			int comp_len = compressed_bytes == null ? length+HEADER_LEN : (int)compressed_bytes.Length;
+			int ucomp_len = compressed_bytes == null ? 0 : length+HEADER_LEN;
 
-			// make sure we are using a packet prep'ed for compression
-			// and that our packet is large enough to warrant compression
-			// re: my_compress.c from mysql src
-			int offset = HEADER_LEN + COMPRESS_HEADER_LEN;
-			int original_len = (int)(_packet.Length - offset);
-			if (original_len < MIN_COMPRESS_LEN) return 0;
-
-			byte[] packetData = _packet.ToArray();
-
-			byte[] output = new Byte[ original_len * 2 ];
-			Deflater d = new Deflater();
-			d.SetInput( packetData, offset, original_len );
-			d.Finish();
-			int comp_len = d.Deflate( output, offset, output.Length - offset  );
-
-			if (comp_len > original_len) return 0;
-			_packet = new MemoryStream( output, 0, comp_len + offset );
-			return (int)comp_len;
-		}
-*/
-		protected byte[] CompressPacket(Packet packet)
-		{
-			if (packet.Length < MIN_COMPRESS_LENGTH) return null;
-
-			byte[] compressed_buffer = new byte[packet.Length * 2];
-			Deflater deflater = new Deflater();
-			deflater.SetInput( packet.GetBuffer(), 0, packet.Length );
-			deflater.Finish();
-			int comp_len = deflater.Deflate( compressed_buffer, 0, compressed_buffer.Length );
-			if (comp_len > packet.Length) return null;
-			return compressed_buffer;
-		}
-
-		protected void SendPacket(Packet packet)
-		{
-			Packet header = null;
-			byte[] buffer = null;
-
-			if (useCompression)
-			{
-				byte[] compressed_bytes = CompressPacket(packet);
-				header = new Packet();
-				
-				// if we succeeded in compressing
-				if (compressed_bytes != null) 
-				{
-					header.WriteInteger( compressed_bytes.Length, 3 );
-					header.WriteByte( packetSeq );
-					header.WriteInteger( packet.Length + HEADER_LEN, 3 );
-					buffer = compressed_bytes;
-				}
-				else
-				{
-					header.WriteInteger( packet.Length + HEADER_LEN, 3 );
-					header.WriteByte( packetSeq );
-					header.WriteInteger( 0, 3 );
-					buffer = packet.GetBuffer();
-				}
-				// now write the internal header
-				header.WriteInteger( packet.Length, 3 );
-				header.WriteByte( 0 );
-			}
+			WriteInteger( comp_len, 3 );
+			writer.WriteByte( packetSeq++ );
+			WriteInteger( ucomp_len, 3 );
+			if (compressed_bytes != null)
+				writer.Write( compressed_bytes.GetBuffer(), 0, (int)compressed_bytes.Length );
 			else 
 			{
-				buffer = packet.GetBytes( packetSeq );
+				WriteInteger( length, 3 );	
+				writer.WriteByte( 0 );
+				writer.Write( buf, index, length );
 			}
-			packetSeq++;
-
-			// send the data to eth server
-			int start = 0;
-			if (! useCompression) start = 3;
-			stream.Write( buffer, start, packet.Length - start ); 
 			stream.Flush();
 		}
 
+		protected void SendBuffer( byte[] buf, int offset, int length )
+		{
+			while (length > 0)
+			{
+				int amount = Math.Min( 1024, length );
+				writer.Write( buf, offset, amount );
+				writer.Flush();
+				offset += amount;
+				length -= amount;
+			}
+		}
+
+		/// <summary>
+		/// Send a single packet to the server.
+		/// </summary>
+		/// <param name="packet">Packet to send to the server</param>
+		/// <remarks>This method will send a single packet to the server
+		/// possibly breaking the packet up into smaller packets that are
+		/// smaller than max_allowed_packet.  This method will always send at
+		/// least one packet to the server</remarks>
+        protected void SendPacket(Packet packet)
+		{
+			byte[]	buf = packet.GetBuffer();
+			int		len = packet.Length;
+			int		index = 0;
+			bool	oneSent = false;
+
+			// make sure we are not trying to send too much
+			if (packet.Length > maxPacketSize && maxPacketSize > 0)
+				throw new MySqlException("Packet size too large.  This MySQL server cannot accept rows larger than " + maxPacketSize + " bytes.");
+
+			try 
+			{
+				while (len > 0 || ! oneSent) 
+				{
+					int lenToSend = Math.Min( len, MAX_PACKET_SIZE );
+
+					// send the data
+					if (useCompression)
+						SendCompressedBuffer( buf, index, lenToSend );
+					else 
+					{
+						WriteInteger( lenToSend, 3 );
+						writer.WriteByte( packetSeq++ );
+						writer.Write( buf, index, lenToSend );
+						writer.Flush();
+					}
+
+					len -= lenToSend;
+					index += lenToSend;
+					oneSent = true;
+				}
+				writer.Flush();
+			}
+			catch (Exception ex)
+			{
+				Console.WriteLine( ex.Message );
+			}
+		}
+
 
 		public void Close() 
 		{
@@ -373,49 +433,86 @@ namespace ByteFX.Data.MySqlClient
 		/// <param name="command">Command to execute</param>
 		/// <param name="text">Text attribute of command</param>
 		/// <returns>Result packet returned from database server</returns>
-		public void SendCommand( DBCmd command, String text ) 
+		public void Send( DBCmd command, String text ) 
 		{
-			Packet packet = new Packet();
-			packetSeq = 0;
-			packet.WriteByte( (byte)command );
-			packet.WriteStringNoNull( text, encoding );
-			SendPacket(packet);
-			
-			packet = ReadPacket();
-			if (packet[0] != 0)
+			CommandResult result = Send( command, this.Encoding.GetBytes( text ) );
+			if (result.IsResultSet)
 				throw new MySqlException("SendCommand failed for command " + text );
 		}
 
-		/// <summary>
-		/// SendQuery sends a byte array of SQL to the server
-		/// </summary>
-		/// <param name="sql"></param>
-		/// <returns>A packet containing the bytes returned by the server</returns>
-		public Packet SendQuery( byte[] sql )
+		public CommandResult Send( DBCmd cmd, byte[] bytes )
 		{
-			string s = encoding.GetString(sql);
+//			string s = Encoding.GetString( bytes );
 
-			Packet packet = new Packet();
+			Packet packet = CreatePacket(null);
 			packetSeq = 0;
-			packet.WriteByte( (byte)DBCmd.QUERY );
-			packet.Write( sql, 0, sql.Length );
-
-			SendPacket( packet );
-			return ReadPacket();
-		}
-
-		
-
-		public Packet SendSql( byte[] bytes )
-		{
-			Packet packet = new Packet();
-			packetSeq = 0;
-			packet.WriteByte( (byte)DBCmd.QUERY );
-			packet.Write( bytes, 0, bytes.Length );
+			packet.WriteByte( (byte)cmd );
+			if (bytes != null)
+				packet.Write( bytes, 0, bytes.Length );
 
 			SendPacket( packet );
 			packet = ReadPacket();
-			return packet;
+
+			// first check to see if this is a LOAD DATA LOCAL callback
+			// if so, send the file and then read the results
+			long fieldcount = packet.ReadLenInteger();
+			if (fieldcount == Packet.NULL_LEN)
+			{
+				string filename = packet.ReadString();
+				SendFileToServer( filename );
+				packet = ReadPacket();
+			}
+			else
+				packet.Position = 0;
+
+			return new CommandResult(packet, this);
+		}
+
+		/// <summary>
+		/// Sends the specified file to the server. 
+		/// This supports the LOAD DATA LOCAL INFILE
+		/// </summary>
+		/// <param name="filename"></param>
+		private void SendFileToServer( string filename )
+		{
+			Packet		p = CreatePacket(null);
+			byte[]		buffer = new byte[4092];
+			FileStream	fs = null;
+			try 
+			{
+				fs = new FileStream( filename, FileMode.Open );
+				int count = fs.Read( buffer, 0, buffer.Length );
+				while (count != 0) 
+				{
+					if ((p.Length + count) > MAX_PACKET_SIZE)
+					{
+						SendPacket( p );
+						p.Clear();
+					}
+					p.Write( buffer, 0, count );
+					count = fs.Read( buffer, 0, buffer.Length );
+				}
+				fs.Close();
+
+				// send any remaining data
+				if (p.Length > 0) 
+				{
+					SendPacket(p);
+					p.Clear();
+				}
+			}
+			catch (Exception ex)
+			{
+				throw new MySqlException("Error during LOAD DATA LOCAL INFILE", ex);
+			}
+			finally 
+			{
+				if (fs != null)
+					fs.Close();
+				// empty packet signals end of file
+				p.Clear();
+				SendPacket(p);
+			}
 		}
 
 		#region PasswordStuff
