@@ -10,6 +10,27 @@
 //
 // This is an OO wrapper API for the Cairo API
 //
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
 
 using System;
 using System.Drawing;
@@ -88,6 +109,11 @@ namespace Cairo {
 			Dispose (false);
 		}
 
+		public void Show (Graphics gr, int width, int height) 
+		{
+			CairoAPI.cairo_show_surface (gr.Handle, surface, width,  height);
+		}
+
 		void IDisposable.Dispose ()
 		{
 			Dispose (true);
@@ -112,7 +138,7 @@ namespace Cairo {
                 public int Repeat {
                         set {
                                 CairoAPI.cairo_surface_set_repeat (surface, value);
-                        }
+                        } 
                 }
 
                 public Cairo.Matrix Matrix {
@@ -121,10 +147,8 @@ namespace Cairo {
                         }
 
                         get {
-                                IntPtr p;
-                                
+                                IntPtr p = IntPtr.Zero;
                                 CairoAPI.cairo_surface_get_matrix (surface, out p);
-
                                 return new Cairo.Matrix (p);
                         }
                 }
