@@ -1,3 +1,26 @@
+
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
 using System;
 using System.Collections;
 
@@ -101,11 +124,30 @@ namespace System.Data.Common
 			return (_nullValues != null) ? _nullValues[index] : true;
 		}
 
-		protected void SetNull(int index,bool isNull)
+		internal void SetNullBit(int index,bool isNull)
 		{
 			_nullValues[index] = isNull;
 		}
 
+		protected void SetNull(int index,bool isNull,bool isDbNull)
+		{
+			SetNullBit(index,isDbNull);
+			// this method must be called after setting the value into value array
+			// otherwise the dafault value will be overriden
+			if ( isNull ) {
+				// set the value to default
+				CopyValue(Column.Table.DefaultValuesRowIndex,index);
+			}
+		}
+
+		internal void FillValues(int fromIndex)
+		{
+			for(int i=0; i < Capacity; i++) {
+				CopyValue(fromIndex,i);
+				_nullValues[i] = _nullValues[fromIndex];
+			}
+		}
+
 		internal virtual void CopyValue(AbstractDataContainer fromContainer, int fromIndex, int toIndex)
 		{
 			_nullValues[toIndex] = fromContainer._nullValues[fromIndex];
@@ -118,7 +160,15 @@ namespace System.Data.Common
 
 		internal virtual void SetItemFromDataRecord(int index, IDataRecord record, int field)
 		{
-			SetNull(index,record.IsDBNull(field));
+			bool isDbNull = record.IsDBNull(field);
+			SetNull(index,false,isDbNull);
+		}
+
+		protected  bool CheckAndSetNull( int index, IDataRecord record, int field)
+		{
+			 bool isDbNull = record.IsDBNull(field);
+	    	         SetNull(index,false,isDbNull);
+			 return isDbNull;		
 		}
 
 		protected int CompareNulls(int index1, int index2)
@@ -136,6 +186,8 @@ namespace System.Data.Common
 
 		internal abstract int CompareValues(int index1, int index2);
 
+		internal abstract long GetInt64(int index);
+
 		#endregion //Methods
 
 		sealed class Int16DataContainer : AbstractDataContainer
@@ -168,7 +220,7 @@ namespace System.Data.Common
 					else {
 						SetValue(index,Convert.ToInt16(value));
 					}
-					SetNull(index,isDbNull);
+					SetNull(index,value == null,isDbNull);
 				}
 			}
 
@@ -199,8 +251,8 @@ namespace System.Data.Common
 			{
 				// if exception thrown, it should be caught 
 				// in the  caller method
-				SetValue(index,record.GetInt16(field));
-				base.SetItemFromDataRecord(index,record,field);
+				if (!CheckAndSetNull ( index, record,field))
+					SetValue(index,record.GetInt16(field));
 			}
 
 			internal override void CopyValue(int fromIndex, int toIndex)
@@ -242,6 +294,11 @@ namespace System.Data.Common
 				return 1;
 			}
 
+			internal override long GetInt64(int index)
+			{
+				return (long) _values[index];
+			}
+
 			#endregion //Methods
 		}
 
@@ -275,7 +332,7 @@ namespace System.Data.Common
 					else {
 						SetValue(index,Convert.ToInt32(value));
 					}
-					SetNull(index,isDbNull);
+					SetNull(index,value == null,isDbNull);
 				}
 			}
 
@@ -306,8 +363,8 @@ namespace System.Data.Common
 			{
 				// if exception thrown, it should be caught 
 				// in the  caller method
-				SetValue(index,record.GetInt32(field));
-				base.SetItemFromDataRecord(index,record,field);
+				if (!CheckAndSetNull ( index, record,field))
+                                        SetValue(index,record.GetInt32(field));
 			}
 
 			internal override void CopyValue(int fromIndex, int toIndex)
@@ -349,6 +406,11 @@ namespace System.Data.Common
 				return 1;
 			}
 
+			internal override long GetInt64(int index)
+			{
+				return (long) _values[index];
+			}
+
 			#endregion //Methods
 		}
 
@@ -382,7 +444,7 @@ namespace System.Data.Common
 					else {
 						SetValue(index,Convert.ToInt64(value));
 					}
-					SetNull(index,isDbNull);
+					SetNull(index,value == null,isDbNull);
 				}
 			}
 
@@ -413,8 +475,8 @@ namespace System.Data.Common
 			{
 				// if exception thrown, it should be caught 
 				// in the  caller method
-				SetValue(index,record.GetInt64(field));
-				base.SetItemFromDataRecord(index,record,field);
+				if (!CheckAndSetNull ( index, record,field))
+                                        SetValue(index,record.GetInt64(field));
 			}
 
 			internal override void CopyValue(int fromIndex, int toIndex)
@@ -447,6 +509,11 @@ namespace System.Data.Common
 				return 1;
 			}
 
+			internal override long GetInt64(int index)
+			{
+				return _values[index];
+			}
+
 			#endregion //Methods
 		}
 
@@ -480,7 +547,7 @@ namespace System.Data.Common
 					else {
 						SetValue(index,Convert.ToSingle(value));
 					}
-					SetNull(index,isDbNull);
+					SetNull(index,value == null,isDbNull);
 				}
 			}
 
@@ -511,8 +578,8 @@ namespace System.Data.Common
 			{
 				// if exception thrown, it should be caught 
 				// in the  caller method
-				SetValue(index,record.GetFloat(field));
-				base.SetItemFromDataRecord(index,record,field);
+				if (!CheckAndSetNull ( index, record,field))
+                                        SetValue(index,record.GetFloat(field));
 			}
 
 			internal override void CopyValue(int fromIndex, int toIndex)
@@ -545,6 +612,11 @@ namespace System.Data.Common
 				return 1;
 			}
 
+			internal override long GetInt64(int index)
+			{
+				return Convert.ToInt64(_values[index]);
+			}
+
 			#endregion //Methods
 		}
 
@@ -578,7 +650,7 @@ namespace System.Data.Common
 					else {
 						SetValue(index,Convert.ToDouble(value));
 					}
-					SetNull(index,isDbNull);
+					SetNull(index,value == null,isDbNull);
 				}
 			}
 
@@ -609,8 +681,8 @@ namespace System.Data.Common
 			{
 				// if exception thrown, it should be caught 
 				// in the  caller method
-				SetValue(index,record.GetDouble(field));
-				base.SetItemFromDataRecord(index,record,field);
+				if (!CheckAndSetNull ( index, record,field))
+                                        SetValue(index,record.GetDouble(field));
 			}
 
 			internal override void CopyValue(int fromIndex, int toIndex)
@@ -643,6 +715,11 @@ namespace System.Data.Common
 				return 1;
 			}
 
+			internal override long GetInt64(int index)
+			{
+				return Convert.ToInt64(_values[index]);
+			}
+
 			#endregion //Methods
 		}
 
@@ -676,7 +753,7 @@ namespace System.Data.Common
 					else {
 						SetValue(index,Convert.ToByte(value));
 					}
-					SetNull(index,isDbNull);
+					SetNull(index,value == null,isDbNull);
 				}
 			}
 
@@ -707,8 +784,8 @@ namespace System.Data.Common
 			{
 				// if exception thrown, it should be caught 
 				// in the  caller method
-				SetValue(index,record.GetByte(field));
-				base.SetItemFromDataRecord(index,record,field);
+				if (!CheckAndSetNull ( index, record,field))
+                                        SetValue(index,record.GetByte(field));
 			}
 
 			internal override void CopyValue(int fromIndex, int toIndex)
@@ -741,6 +818,11 @@ namespace System.Data.Common
 				return 1;
 			}
 
+			internal override long GetInt64(int index)
+			{
+				return (long) _values[index];
+			}
+
 			#endregion //Methods
 		}
 
@@ -775,7 +857,7 @@ namespace System.Data.Common
 					else {
 						SetValue(index,Convert.ToBoolean(value));
 					}
-					SetNull(index,isDbNull);
+					SetNull(index,value == null,isDbNull);
 				}
 			}
 
@@ -806,12 +888,20 @@ namespace System.Data.Common
 			{
 				// if exception thrown, it should be caught 
 				// in the  caller method
-				SetValue(index,record.GetBoolean(field));
+				if (!CheckAndSetNull ( index, record,field))
+                                        SetValue(index,record.GetBoolean(field));
 			}
 
 			internal override void CopyValue(int fromIndex, int toIndex)
 			{
 				base.CopyValue(fromIndex, toIndex);
+				_values[toIndex] = _values[fromIndex];
+			}
+
+			internal override void CopyValue(AbstractDataContainer fromContainer, int fromIndex, int toIndex)
+			{
+				base.CopyValue(fromContainer, fromIndex, toIndex);
+				_values[toIndex] = ((BitDataContainer)fromContainer)._values[fromIndex];
 			}
 
 			internal override int CompareValues(int index1, int index2)
@@ -830,6 +920,11 @@ namespace System.Data.Common
 				return CompareNulls(index1, index2);	
 			}
 
+			internal override long GetInt64(int index)
+			{
+				return Convert.ToInt64(_values[index]);
+			}
+
 			#endregion //Methods
 		}
 
@@ -849,7 +944,7 @@ namespace System.Data.Common
 				}
 				set {
 					SetValue(index,value);
-					SetNull(index,value == DBNull.Value);
+					SetNull(index,value == null,value == DBNull.Value);
 				}
 			}
 
@@ -882,7 +977,7 @@ namespace System.Data.Common
 			internal override void SetItemFromDataRecord(int index, IDataRecord record, int field)
 			{
 				// if exception thrown, it should be caught 
-				// in the  caller method
+				// in the  caller metho
 				SetValue(index,record.GetValue(field));
 				base.SetItemFromDataRecord(index,record,field);
 			}
@@ -923,6 +1018,11 @@ namespace System.Data.Common
 				return String.Compare(obj1.ToString(), obj2.ToString());
 			}
 
+			internal override long GetInt64(int index)
+			{
+				return Convert.ToInt64(_values[index]);
+			}
+
 			#endregion //Methods
 	 
 		}
@@ -958,8 +1058,8 @@ namespace System.Data.Common
 			{
 				// if exception thrown, it should be caught 
 				// in the  caller method
-				SetValue(index,record.GetString(field));
-				base.SetItemFromDataRecord(index,record,field);
+				if (!CheckAndSetNull ( index, record,field))
+                                        SetValue(index,record.GetString(field));
 			}
 
 			internal override int CompareValues(int index1, int index2)
@@ -998,8 +1098,8 @@ namespace System.Data.Common
 			{
 				// if exception thrown, it should be caught 
 				// in the  caller method
-				base.SetValue(index,record.GetDateTime(field));
-				base.SetItemFromDataRecord(index,record,field);
+				if (!CheckAndSetNull(index,record,field))
+					base.SetValue(index,record.GetDateTime(field));
 			}
 
 			internal override int CompareValues(int index1, int index2)
