@@ -7,6 +7,29 @@
 // Copyright (C) Brian Ritchie, 2002
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Data;
 using System.Data.Common;
@@ -79,6 +102,7 @@ namespace System.Data.Odbc
 			this.sourceColumn = srcColumn;
 		}
 
+		[EditorBrowsable (EditorBrowsableState.Advanced)]
 		public OdbcParameter(string name, OdbcType dataType, int size, ParameterDirection direction, bool isNullable, byte precision, byte scale, string srcColumn, DataRowVersion srcVersion, object value)
 			: this (name, dataType, size, srcColumn)
 		{
@@ -142,36 +166,57 @@ namespace System.Data.Odbc
 			}
 		}
 		
+ 		[OdbcDescription ("DataParameter_ParameterName")]
+                [DefaultValue ("")]	
 		public string ParameterName {
 			get { return name; }
 			set { name = value; }
 		}
 
+		[OdbcDescription ("DbDataParameter_Precision")]
+                [OdbcCategory ("DataCategory_Data")]
+                [DefaultValue (0)]
 		public byte Precision {
 			get { return precision; }
 			set { precision = value; }
 		}
 		
+                [OdbcDescription ("DbDataParameter_Scale")]
+                [OdbcCategory ("DataCategory_Data")]
+                [DefaultValue (0)]
 		public byte Scale {
 			get { return scale; }
 			set { scale = value; }
 		}
 		
+		[OdbcDescription ("DbDataParameter_Size")]
+                [OdbcCategory ("DataCategory_Data")]
+                [DefaultValue (0)]
 		public int Size {
 			get { return size; }
 			set { size = value; }
 		}
 
+		[OdbcDescription ("DataParameter_SourceColumn")]
+                [OdbcCategory ("DataCategory_Data")]
+                [DefaultValue ("")]
 		public string SourceColumn {
 			get { return sourceColumn; }
 			set { sourceColumn = value; }
 		}
 		
+                [OdbcDescription ("DataParameter_SourceVersion")]
+                [OdbcCategory ("DataCategory_Data")]
+                [DefaultValue (512)]			
 		public DataRowVersion SourceVersion {
 			get { return sourceVersion; }
 			set { sourceVersion = value; }
 		}
-		
+
+		[TypeConverter (typeof(StringConverter))]
+                [OdbcDescription ("DataParameter_Value")]
+                [OdbcCategory ("DataCategory_Data")]
+                [DefaultValue (null)]		
 		public object Value {
 			get { 
 				return ParamValue;
@@ -216,7 +261,10 @@ namespace System.Data.Odbc
 				string paramValueString = ParamValue.ToString();
 				// Treat everything else as a string
 				// Init string buffer
-				if (buffer == null || buffer.Length < ((size > 20) ? size : 20))
+				 if (ParamValue is String)
+                                        paramValueString = "\'"+paramValueString+"\'";
+                                 
+				 if (buffer == null || buffer.Length < ((size > 20) ? size : 20))
 					buffer = new byte[(size > 20) ? size : 20];
 				else
 					buffer.Initialize();
