@@ -14,6 +14,29 @@
 // Copyright (C) Phillip Jerkins, 2003
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using Mono.Data.Tds;
 using Mono.Data.Tds.Protocol;
 using System;
@@ -393,6 +416,9 @@ namespace System.Data.SqlClient {
 		{
 			theServerName = "";
 			string theInstanceName = "";
+			if ((theDataSource == null) || (theServerName == null))
+				throw new ArgumentException("Format of initialization string doesnot conform to specifications");
+				
 			thePort = 1433; // default TCP port for SQL Server
 			bool success = true;
                         			
@@ -433,7 +459,7 @@ namespace System.Data.SqlClient {
                 {
 			NameValueCollection parameters = new NameValueCollection ();
 
-			if (connectionString.Length == 0)
+			if (( connectionString == null)||( connectionString.Length == 0)) 
 				return;
 			connectionString += ";";
 
@@ -522,8 +548,8 @@ namespace System.Data.SqlClient {
 
                         this.connectionString = connectionString;
                 }
-
-                void SetDefaultConnectionParameters (NameValueCollection parameters)
+	
+	        void SetDefaultConnectionParameters (NameValueCollection parameters)
                 {
                         if (null == parameters.Get ("APPLICATION NAME"))
                                 parameters["APPLICATION NAME"] = "Mono SqlClient Data Provider";
@@ -632,7 +658,9 @@ namespace System.Data.SqlClient {
                                 case "WORKSTATION ID" :
                                         parms.Hostname = value;
                                         break;
-                                }
+				default :
+					throw new ArgumentException("Keyword not supported :"+name);
+			          }
 			}
 		}
 
