@@ -8,6 +8,29 @@
 // (C) Copyright 2002 Tim Coleman
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Globalization;
 
@@ -20,6 +43,7 @@ namespace System.Data.SqlTypes
 	        Guid value;
 
 		private bool notNull;
+		private byte[] lastSixBytes;
 
 		public static readonly SqlGuid Null;
 
@@ -29,6 +53,7 @@ namespace System.Data.SqlTypes
 
 		public SqlGuid (byte[] value) 
 		{
+			lastSixBytes = new byte[6];
 			this.value = new Guid (value);
 			notNull = true;
 		}
@@ -36,18 +61,21 @@ namespace System.Data.SqlTypes
 		public SqlGuid (Guid g) 
 		{
 			this.value = g;
+			lastSixBytes = new byte[6];
 			notNull = true;
 		}
 
 		public SqlGuid (string s) 
 		{
 			this.value = new Guid (s);
+			lastSixBytes = new byte[6];
 			notNull = true;
 		}
 
 		public SqlGuid (int a, short b, short c, byte d, byte e, byte f, byte g, byte h, byte i, byte j, byte k) 
 		{
 			this.value = new Guid (a, b, c, d, e, f, g, h, i, j, k);
+			lastSixBytes = new byte[6];
 			notNull = true;
 		}
 
@@ -68,6 +96,21 @@ namespace System.Data.SqlTypes
 			}
 		}
 
+		private byte[] GetLastSixBytes()
+		{
+			lastSixBytes = new byte[6];
+			
+			byte[] guidArray = value.ToByteArray();
+			lastSixBytes[0] = guidArray[10];
+			lastSixBytes[1] = guidArray[11];
+			lastSixBytes[2] = guidArray[12];
+			lastSixBytes[3] = guidArray[13];
+			lastSixBytes[4] = guidArray[14];
+			lastSixBytes[5] = guidArray[15];
+		
+			return lastSixBytes;
+		}	
+
 		#endregion
 
 		#region Methods
@@ -81,9 +124,34 @@ namespace System.Data.SqlTypes
 			else if (((SqlGuid)value).IsNull)
 				return 1;
 			else
-				return this.value.CompareTo (((SqlGuid)value).Value);
+				{
+					//MSDN documentation says that CompareTo is different from 
+					//Guid's CompareTo. It uses the SQL Server behavior where					    //only the last 6 bytes of value are evaluated	
+					byte[] compareValue = ((SqlGuid)value).GetLastSixBytes();
+					byte[] currentValue = GetLastSixBytes();
+					for (int i = 0; i < 6; i++)
+					{
+						if (currentValue[i] != compareValue[i]) {
+			                              return Compare(currentValue[i], compareValue[i]);
+                        			}
+					}
+			                return 0;
+				}
+				
 		}
 
+
+		private static int Compare (uint x, uint y)
+                {
+                        if (x < y) {
+                                return -1;
+                        }
+                        else {
+                                return 1;
+                        }
+                }
+
+
 		public override bool Equals (object value)
 		{
 			if (!(value is SqlGuid))
