@@ -8,6 +8,29 @@
 // (C) Copyright 2002 Tim Coleman
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Globalization;
 
@@ -34,23 +57,7 @@ namespace System.Data.SqlTypes
 		{
 			if (value > 922337203685477.5807m || value < -922337203685477.5808m)
 				throw new OverflowException ();
-
-			value = Decimal.Round (value, 4);
-
-			int [] bits = Decimal.GetBits (value);
-			int scaleDiff = 4 - ((bits [3] & 0x7FFF0000) >> 16);
-			decimal tmp = value;
-			// integrify
-			if (scaleDiff > 0)
-				for (int i = 0; i < scaleDiff; i++)
-					tmp *= 10;
-			else if (scaleDiff < 0)
-				for (int i = 0; i > scaleDiff; i--)
-					tmp /= 10;
-			int [] tmpbits = decimal.GetBits (tmp);
-			tmpbits [3] = (value < 0) ? 0x8004 << 16 : 0x40000;
-			this.value = new decimal (tmpbits);
-
+			this.value = Decimal.Round (value, 4);
 			notNull = true;
 		}
 
@@ -98,12 +105,24 @@ namespace System.Data.SqlTypes
 				return 1;
 			else if (!(value is SqlMoney))
 				throw new ArgumentException (Locale.GetText ("Value is not a System.Data.SqlTypes.SqlMoney"));
-			else if (((SqlMoney)value).IsNull)
+			return CompareSqlMoney ((SqlMoney)value);
+		}
+		
+		private int CompareSqlMoney (SqlMoney value)
+		{
+			if (value.IsNull)
 				return 1;
 			else
-				return this.value.CompareTo (((SqlMoney)value).Value);
+				return this.value.CompareTo (value.Value);
 		}
 
+		#if NET_2_0
+		public int CompareTo (SqlMoney value)
+		{
+			return CompareSqlMoney (value);
+		}
+		#endif
+
 		public static SqlMoney Divide (SqlMoney x, SqlMoney y)
 		{
 			return (x / y);
