@@ -11,6 +11,29 @@
 // Copyright (C) Daniel Morgan, 2003
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections;
 using System.ComponentModel;
@@ -103,20 +126,9 @@ namespace System.Data {
 		/// <returns></returns>
 		public virtual DataColumn Add()
 		{
-			//FIXME:
 			string defaultName = GetNextDefaultColumnName ();
 			DataColumn column = new DataColumn (defaultName);
-			CollectionChangeEventArgs e = new CollectionChangeEventArgs(CollectionChangeAction.Add, this);
-			
-			column.SetTable(parentTable);
-			RegisterName(defaultName,column);
-			base.List.Add(column);
-			
-			if (column.AutoIncrement)
-				autoIncrement.Add(defaultName);
-
-			column.SetOrdinal(Count - 1);
-			OnCollectionChanged(e);
+			Add (column);
 			return column;
 		}
 
@@ -214,7 +226,8 @@ namespace System.Data {
 			}
 
 			if (column.AutoIncrement)
-				autoIncrement.Add(column.ColumnName);
+				autoIncrement.Add(column);
+
 			OnCollectionChanged (e);
 		}
 
@@ -507,12 +520,12 @@ namespace System.Data {
 			{
 				this[i].SetOrdinal( i );
 			}
-			
+
 			if (parentTable != null)
 				parentTable.OnRemoveColumn(column);
 
 			if (column.AutoIncrement)
-				autoIncrement.Remove(column.ColumnName);
+				autoIncrement.Remove(column);
 
 			OnCollectionChanged(e);
 			return;
@@ -562,13 +575,13 @@ namespace System.Data {
 		{
 			if (isAutoIncrement)
 			{
-				if (!autoIncrement.Contains(col.ColumnName))
-					autoIncrement.Add(col.ColumnName);
+				if (!autoIncrement.Contains(col))
+					autoIncrement.Add(col);
 			}
 			else
 			{
-				if (autoIncrement.Contains(col.ColumnName))
-					autoIncrement.Remove(col.ColumnName);
+				if (autoIncrement.Contains(col))
+					autoIncrement.Remove(col);
 			}
 		}
 
