@@ -14,6 +14,29 @@
 // Copyright (C) Tim Coleman, 2002, 2003
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections;
 using System.ComponentModel;
@@ -890,7 +913,8 @@ namespace System.Data {
 				}
 			}
 			// Otherwise, read as dataset... but only when required.
-			bool inferedSchema = false;
+			XmlReadMode explicitReturnMode = XmlReadMode.Auto;
+			XmlDocument doc;
 			switch (mode) {
 			case XmlReadMode.Auto:
 				if (Tables.Count > 0)
@@ -898,8 +922,7 @@ namespace System.Data {
 				else
 					goto case XmlReadMode.InferSchema;
 			case XmlReadMode.InferSchema:
-#if true // sync with the switch immediately below
-				XmlDocument doc = new XmlDocument ();
+				doc = new XmlDocument ();
 				do {
 					doc.AppendChild (doc.ReadNode (reader));
 					reader.MoveToContent ();
@@ -908,8 +931,24 @@ namespace System.Data {
 				} while (!reader.EOF);
 				InferXmlSchema (doc, null);
 				reader = new XmlNodeReader (doc);
-#endif
-				inferedSchema = true;
+				explicitReturnMode = XmlReadMode.InferSchema;
+				break;
+			case XmlReadMode.ReadSchema:
+				doc = new XmlDocument ();
+				do {
+					doc.AppendChild (doc.ReadNode (reader));
+					reader.MoveToContent ();
+					if (doc.DocumentElement != null)
+						break;
+				} while (!reader.EOF);
+				if (doc.DocumentElement != null) {
+					XmlElement schema = doc.DocumentElement ["schema", XmlSchema.Namespace] as XmlElement;
+					if (schema != null) {
+						ReadXmlSchema (new XmlNodeReader (schema));
+						explicitReturnMode = XmlReadMode.ReadSchema;
+					}
+				}
+				reader = new XmlNodeReader (doc);
 				break;
 			case XmlReadMode.IgnoreSchema:
 			case XmlReadMode.Fragment:
@@ -918,15 +957,11 @@ namespace System.Data {
 				reader.Skip ();
 				return mode;
 			}
-#if true // sync with the switch immediately above
+
 			XmlDataReader.ReadXml (this, reader, mode);
-			if (inferedSchema)
-				return XmlReadMode.InferSchema;
+			if (explicitReturnMode != XmlReadMode.Auto)
+				return explicitReturnMode;
 			return mode == XmlReadMode.Auto ? XmlReadMode.IgnoreSchema : mode;
-#else
-			XmlDataLoader Loader = new XmlDataLoader (this);
-			return Loader.LoadData (reader, mode);
-#endif
 		}
 		#endregion // Public Methods
 
@@ -938,14 +973,6 @@ namespace System.Data {
 
 		#endregion // Public Events
 
-		#region Destructors
-
-		~DataSet ()
-		{
-		}
-
-		#endregion Destructors
-
 		#region IListSource methods
 		IList IListSource.GetList ()
 		{
@@ -1059,7 +1086,7 @@ namespace System.Data {
 		{
 		}
 
-		protected internal virtual void OnMergeFailed (MergeFailedEventArgs e)
+		internal virtual void OnMergeFailed (MergeFailedEventArgs e)
 		{
 			if (MergeFailed != null)
 				MergeFailed (this, e);
@@ -1117,13 +1144,14 @@ namespace System.Data {
 			//part of another table structure via a nested parent relationship
 			foreach (DataTable table in tableCollection) {
 				bool isTopLevel = true;
+				/*
 				foreach (DataRelation rel in table.ParentRelations) {
 					if (rel.Nested) {
 						isTopLevel = false;
 						break;
 					}
 				}
-				
+				*/
 				if (isTopLevel) {
 					WriteTable ( writer, table, mode, version);
 				}
@@ -1134,10 +1162,10 @@ namespace System.Data {
 		{
 			DataRow[] rows = new DataRow [table.Rows.Count];
 			table.Rows.CopyTo (rows, 0);
-			WriteTable (writer, rows, mode, version);
+			WriteTable (writer, rows, mode, version, true);
 		}
 
-		private void WriteTable (XmlWriter writer, DataRow[] rows, XmlWriteMode mode, DataRowVersion version)
+		private void WriteTable (XmlWriter writer, DataRow[] rows, XmlWriteMode mode, DataRowVersion version, bool skipIfNested)
 		{
 			//The columns can be attributes, hidden, elements, or simple content
 			//There can be 0-1 simple content cols or 0-* elements
@@ -1150,8 +1178,32 @@ namespace System.Data {
 			SplitColumns (table, out atts, out elements, out simple);
 			//sort out the namespacing
 			string nspc = table.Namespace.Length > 0 ? table.Namespace : Namespace;
+			int relationCount = table.ParentRelations.Count;
+			DataRelation oneRel = relationCount == 1 ? table.ParentRelations [0] : null;
 
 			foreach (DataRow row in rows) {
+				if (skipIfNested) {
+					// Skip rows that is a child of any tables.
+					switch (relationCount) {
+					case 0:
+						break;
+					case 1:
+						if (row.GetParentRow (oneRel) != null)
+							continue;
+						break;
+					case 2:
+						bool skip = false;
+						for (int i = 0; i < table.ParentRelations.Count; i++)
+							if (row.GetParentRow (table.ParentRelations [i]) != null) {
+								skip = true;
+								continue;
+							}
+						if (skip)
+							continue;
+						break;
+					}
+				}
+
 				if (!row.HasVersion(version) || 
 				   (mode == XmlWriteMode.DiffGram && row.RowState == DataRowState.Unchanged 
 				      && version == DataRowVersion.Original))
@@ -1190,7 +1242,7 @@ namespace System.Data {
 				
 				foreach (DataRelation relation in table.ChildRelations) {
 					if (relation.Nested) {
-						WriteTable (writer, row.GetChildRows (relation), mode, version);
+						WriteTable (writer, row.GetChildRows (relation), mode, version, false);
 					}
 				}
 				
@@ -1218,7 +1270,7 @@ namespace System.Data {
 
 		private void WriteColumnAsAttribute (XmlWriter writer, XmlWriteMode mode, DataColumn col, DataRow row, DataRowVersion version)
 		{
-			WriteAttributeString (writer, mode, col.Namespace, col.Prefix, XmlConvert.EncodeLocalName (col.ColumnName), row[col, version].ToString ());
+			WriteAttributeString (writer, mode, col.Namespace, col.Prefix, XmlConvert.EncodeLocalName (col.ColumnName), WriteObjectXml (row[col, version]));
 		}
 
 		private void WriteTableElement (XmlWriter writer, XmlWriteMode mode, DataTable table, DataRow row, DataRowVersion version)
@@ -1230,7 +1282,7 @@ namespace System.Data {
 
 			if (mode == XmlWriteMode.DiffGram) {
 				WriteAttributeString (writer, mode, XmlConstants.DiffgrNamespace, XmlConstants.DiffgrPrefix, "id", table.TableName + (row.XmlRowID + 1));
-				WriteAttributeString (writer, mode, XmlConstants.MsdataNamespace, XmlConstants.MsdataPrefix, "rowOrder", row.XmlRowID.ToString());
+				WriteAttributeString (writer, mode, XmlConstants.MsdataNamespace, XmlConstants.MsdataPrefix, "rowOrder", XmlConvert.ToString (row.XmlRowID));
 				string modeName = null;
 				if (row.RowState == DataRowState.Modified)
 					modeName = "modified";
@@ -1287,13 +1339,14 @@ namespace System.Data {
 			}
 			writer.WriteEndElement (); // DataSet name or diffgr:diffgram
 		}
+		
 
 		private void CheckNamespace (string prefix, string ns, XmlNamespaceManager nsmgr, XmlSchema schema)
 		{
 			if (ns == String.Empty)
 				return;
 			if (ns != nsmgr.DefaultNamespace) {
-				if (nsmgr.LookupNamespace (prefix) != ns) {
+				if (nsmgr.LookupNamespace (nsmgr.NameTable.Get (prefix)) != ns) {
 					for (int i = 1; i < int.MaxValue; i++) {
 						string p = nsmgr.NameTable.Add ("app" + i);
 						if (!nsmgr.HasNamespace (p)) {
@@ -1337,6 +1390,8 @@ namespace System.Data {
 				nsmgr.AddNamespace (XmlConstants.TnsPrefix, Namespace);
 				nsmgr.AddNamespace (String.Empty, Namespace);
 			}
+			if (CheckExtendedPropertyExists ())
+				nsmgr.AddNamespace (XmlConstants.MspropPrefix, XmlConstants.MspropNamespace);
 
 			if (atts.Count > 0)
 				schema.UnhandledAttributes = atts.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
@@ -1361,6 +1416,13 @@ namespace System.Data {
 			attr.Value = locale.Name;
 			atts.Add (attr);
 
+			// Extended properties
+			foreach (DictionaryEntry de in ExtendedProperties) {
+				attr = doc.CreateAttribute (XmlConstants.MspropPrefix, de.Key.ToString (), XmlConstants.MspropNamespace);
+				attr.Value = de.Value != null ? de.Value.ToString () : String.Empty;
+				atts.Add (attr);
+			}
+
 			elem.UnhandledAttributes = atts.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
 
 			XmlSchemaComplexType complex = new XmlSchemaComplexType ();
@@ -1393,38 +1455,57 @@ namespace System.Data {
 
 			schema.Items.Add (elem);
 			
-			AddConstraintsToSchema (elem, constraintPrefix, tables, relations);
+			AddConstraintsToSchema (elem, constraintPrefix, tables, relations, doc);
 			foreach (string prefix in nsmgr) {
-				string ns = nsmgr.LookupNamespace (prefix);
+				string ns = nsmgr.LookupNamespace (nsmgr.NameTable.Get (prefix));
 				if (prefix != "xmlns" && prefix != "xml" && ns != null && ns != String.Empty)
 					schema.Namespaces.Add (prefix, ns);
 			}
 			return schema;
 		}
-		
+
+		private bool CheckExtendedPropertyExists ()
+		{
+			if (ExtendedProperties.Count > 0)
+				return true;
+			foreach (DataTable dt in Tables) {
+				if (dt.ExtendedProperties.Count > 0)
+					return true;
+				foreach (DataColumn col in dt.Columns)
+					if (col.ExtendedProperties.Count > 0)
+						return true;
+				foreach (Constraint c in dt.Constraints)
+					if (c.ExtendedProperties.Count > 0)
+						return true;
+			}
+			foreach (DataRelation rel in Relations)
+				if (rel.ExtendedProperties.Count > 0)
+					return true;
+			return false;
+		}
+
 		// Add all constraints in all tables to the schema.
-		private void AddConstraintsToSchema (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables, DataRelationCollection relations) 
+		private void AddConstraintsToSchema (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables, DataRelationCollection relations, XmlDocument doc) 
 		{
 			// first add all unique constraints.
-			Hashtable uniqueNames = AddUniqueConstraints (elem, constraintPrefix, tables);
+			Hashtable uniqueNames = AddUniqueConstraints (elem, constraintPrefix, tables, doc);
 			// Add all foriegn key constraints.
-			AddForeignKeys (uniqueNames, elem, constraintPrefix, relations);
+			AddForeignKeys (uniqueNames, elem, constraintPrefix, relations, doc);
 		}
 		
 		// Add unique constaraints to the schema.
 		// return hashtable with the names of all XmlSchemaUnique elements we created.
-		private Hashtable AddUniqueConstraints (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables)
+		private Hashtable AddUniqueConstraints (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables, XmlDocument doc)
 		{
-			XmlDocument doc = new XmlDocument();
 			Hashtable uniqueNames = new Hashtable();
 			foreach (DataTable table in tables) {
 				
-				foreach (Constraint constaint in table.Constraints) {
+				foreach (Constraint constraint in table.Constraints) {
 					
-					if (constaint is UniqueConstraint) {
+					if (constraint is UniqueConstraint) {
 						ArrayList attrs = new ArrayList ();
 						XmlAttribute attrib;
-						UniqueConstraint uqConst = (UniqueConstraint)constaint;
+						UniqueConstraint uqConst = (UniqueConstraint) constraint;
 						XmlSchemaUnique uniq = new XmlSchemaUnique ();
 						
 						// if column of the constraint is hidden do not write the constraint.
@@ -1467,6 +1548,8 @@ namespace System.Data {
 							uniq.Fields.Add(field);
 						}
 				
+						AddExtendedPropertyAttributes (uniq, constraint.ExtendedProperties, doc);
+
 						elem.Constraints.Add (uniq);
 						uniqueNames.Add (uniq.Name, null);
 					}
@@ -1476,11 +1559,10 @@ namespace System.Data {
 		}
 		
 		// Add the foriegn keys to the schema.
-		private void AddForeignKeys (Hashtable uniqueNames, XmlSchemaElement elem, string constraintPrefix, DataRelationCollection relations)
+		private void AddForeignKeys (Hashtable uniqueNames, XmlSchemaElement elem, string constraintPrefix, DataRelationCollection relations, XmlDocument doc)
 		{
 			if (relations == null) return;
 			
-			XmlDocument doc = new XmlDocument();
 			foreach (DataRelation rel in relations) {
 				
 				if (rel.ParentKeyConstraint == null || rel.ChildKeyConstraint == null)
@@ -1532,7 +1614,10 @@ namespace System.Data {
 					field.XPath = constraintPrefix+column.ColumnName;
 					keyRef.Fields.Add(field);
 				}
+
 				keyRef.UnhandledAttributes = (XmlAttribute[])attrs.ToArray (typeof (XmlAttribute));
+				AddExtendedPropertyAttributes (keyRef, rel.ExtendedProperties, doc);
+
 				elem.Constraints.Add (keyRef);
 			}
 		}
@@ -1553,7 +1638,6 @@ namespace System.Data {
 
 			XmlSchemaObjectCollection schemaAttributes = null;
 
-			//TODO - what about the simple content?
 			if (simple != null) {
 				// add simpleContent
 				XmlSchemaSimpleContent simpleContent = new XmlSchemaSimpleContent();
@@ -1566,7 +1650,7 @@ namespace System.Data {
 				
 				// add ordinal attribute
 				xlmAttrs[1] = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.Ordinal, XmlConstants.MsdataNamespace);
-				xlmAttrs[1].Value = simple.Ordinal.ToString();
+				xlmAttrs[1].Value = XmlConvert.ToString (simple.Ordinal);
 				simpleContent.UnhandledAttributes = xlmAttrs;
 				
 			        
@@ -1603,12 +1687,12 @@ namespace System.Data {
 
 					if (col.AutoIncrementSeed != 0) {
 						xattr = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.AutoIncrementSeed, XmlConstants.MsdataNamespace);
-						xattr.Value = col.AutoIncrementSeed.ToString();
+						xattr.Value = XmlConvert.ToString (col.AutoIncrementSeed);
 						xattrs.Add (xattr);
 					}
 
 					if (col.DefaultValue.ToString () != String.Empty)
-						colElem.DefaultValue = col.DefaultValue.ToString ();
+						colElem.DefaultValue = WriteObjectXml (col.DefaultValue);
 					
 					if (col.MaxLength < 0)
 						colElem.SchemaTypeName = MapType (col.DataType);
@@ -1635,10 +1719,9 @@ namespace System.Data {
 					}
 					
 					colElem.UnhandledAttributes = (XmlAttribute[])xattrs.ToArray(typeof (XmlAttribute));
+					AddExtendedPropertyAttributes (colElem, col.ExtendedProperties, doc);
 					seq.Items.Add (colElem);
 				}
-				if (seq.Items.Count > 0)
-					complex.Particle = seq;
 
 				foreach (DataRelation rel in table.ChildRelations) {
 					if (rel.Nested) {
@@ -1647,6 +1730,8 @@ namespace System.Data {
 							el.RefName = new XmlQualifiedName (rel.ChildTable.TableName, rel.ChildTable.Namespace);
 						} else {
 							XmlSchemaElement el = GetTableSchema (doc, rel.ChildTable, schemaToAdd, nsmgr);
+							el.MinOccurs = 0;
+							el.MaxOccursString = "unbounded";
 							XmlSchemaComplexType ct = (XmlSchemaComplexType) el.SchemaType;
 							ct.Name = el.Name;
 							el.SchemaType = null;
@@ -1656,6 +1741,9 @@ namespace System.Data {
 						}
 					}
 				}
+
+				if (seq.Items.Count > 0)
+					complex.Particle = seq;
 			}
 
 			//Then a list of attributes
@@ -1671,12 +1759,33 @@ namespace System.Data {
 					schemaToAdd.Namespaces.Add (prefix, col.Namespace);
 				}
 				att.SchemaTypeName = MapType (col.DataType);
+				// FIXME: what happens if extended properties are set on attribute columns??
 				schemaAttributes.Add (att);
 			}
 
+			AddExtendedPropertyAttributes (elem, table.ExtendedProperties, doc);
+
 			return elem;
 		}
 
+		private void AddExtendedPropertyAttributes (XmlSchemaAnnotated xsobj, PropertyCollection props, XmlDocument doc)
+		{
+			ArrayList attList = new ArrayList ();
+			XmlAttribute xmlAttr;
+
+			if (xsobj.UnhandledAttributes != null)
+				attList.AddRange (xsobj.UnhandledAttributes);
+
+			// add extended properties to xs:element
+			foreach (DictionaryEntry de in props) {
+				xmlAttr = doc.CreateAttribute (XmlConstants.MspropPrefix, XmlConvert.EncodeName (de.Key.ToString ()), XmlConstants.MspropNamespace);
+				xmlAttr.Value = de.Value != null ? WriteObjectXml (de.Value) : String.Empty;
+				attList.Add (xmlAttr);
+			}
+			if (attList.Count > 0)
+				xsobj.UnhandledAttributes = attList.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
+		}
+
 		private string SafeNS (string ns)
 		{
 			return ns != null ? ns : String.Empty;
