@@ -8,6 +8,29 @@
 // (C) 2002 Ville Palo
 // (C) 2003 Martin Willemoes Hansen
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using NUnit.Framework;
 using System;
 using System.Data.SqlTypes;
@@ -108,6 +131,12 @@ namespace MonoTests.System.Data.SqlTypes
                         } catch(Exception e) {
                                 AssertEquals ("#D19", typeof (OverflowException), e.GetType ());
                         }
+
+			#if NET_2_0
+			 // Modulus ()
+                        AssertEquals ("#D20", (SqlInt64)36, SqlInt64.Modulus (Test164, Test64));
+                        AssertEquals ("#D21",  (SqlInt64)64, SqlInt64.Modulus (Test64, Test164));
+			#endif
                 }
 
 		[Test]
