@@ -7,8 +7,30 @@
 //
 // (C) Copyright 2002 Ville Palo
 // (C) Copyright 2003 Martin Willemoes Hansen
-// (C) Copyright 2004 Novell Inc.
+
 //
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 
 using NUnit.Framework;
 using System;
@@ -1195,5 +1217,23 @@ namespace MonoTests.System.Data
 			ds.WriteXml (sw);
 			AssertEquals (xml, sw.ToString ().Replace ("\r\n", "\n"));
 		}
+
+		[Test]
+		public void ReadWriteXml2 ()
+		{
+			string xml = "<FullTextResponse><Domains><AvailResponse info='y' name='novell-ximian-group' /><AvailResponse info='n' name='ximian' /></Domains></FullTextResponse>";
+			DataSet ds = new DataSet ();
+			ds.ReadXml (new StringReader (xml));
+			AssertDataSet ("ds", ds, "FullTextResponse", 2, 1);
+			DataTable dt = ds.Tables [0];
+			AssertDataTable ("dt1", dt, "Domains", 1, 1, 0, 1, 1, 1);
+			dt = ds.Tables [1];
+			AssertDataTable ("dt2", dt, "AvailResponse", 3, 2, 1, 0, 1, 0);
+			StringWriter sw = new StringWriter ();
+			XmlTextWriter xtw = new XmlTextWriter (sw);
+			xtw.QuoteChar = '\'';
+			ds.WriteXml (xtw);
+			AssertEquals (xml, sw.ToString ());
+		}
         }
 }
