@@ -8,10 +8,36 @@
 // (C) 2003 Martin Willemoes Hansen
 // 
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using NUnit.Framework;
 using System;
 using System.Data;
 using System.Globalization;
+using System.IO;
+using System.Runtime.Serialization.Formatters.Binary;
+using System.Xml;
 
 namespace MonoTests.System.Data
 {
@@ -1135,18 +1161,21 @@ namespace MonoTests.System.Data
 				table.Reset ();
 				Fail ("#A01");
 			}
-                        catch (ArgumentException) {
+                        catch (InvalidConstraintException) {
 			}
 			try {
 				table.Clear ();
-#if NET_1_1
-#else
+//#if NET_1_1
+//#else
 				Fail ("#A03");
-#endif
+//#endif
 			}
 			catch (Exception e) {
+
 #if NET_1_1
-				throw e;
+				
+				AssertEquals ("#A04", "Cannot clear table Parent because ForeignKeyConstraint DR enforces Child.", e.Message);
+				AssertEquals ("#A11", typeof (InvalidConstraintException), e.GetType());
 #else
 				if (e.GetType () != typeof (AssertionException)) {
 					// FIXME: Don't depend on localizable error messages.
@@ -1163,11 +1192,60 @@ namespace MonoTests.System.Data
 			table.Clear ();
 			AssertEquals ("#A08", 0, table.Rows.Count);
 #if NET_1_1
-			AssertEquals ("#A09", 0, table.Constraints.Count);
+			AssertEquals ("#A09", 1, table.Constraints.Count);
 #else
 			AssertEquals ("#A09", 1, table.Constraints.Count);
 #endif
-			AssertEquals ("#A05", 0, table.ChildRelations.Count);
+			AssertEquals ("#A10", 0, table.ChildRelations.Count);
+		}
+
+		[Test]
+		public void Serialize ()
+		{
+			MemoryStream fs = new MemoryStream ();
+			
+			// Construct a BinaryFormatter and use it 
+			// to serialize the data to the stream.
+			BinaryFormatter formatter = new BinaryFormatter();
+		
+			// Create an array with multiple elements refering to 
+			// the one Singleton object.
+			DataTable dt = new DataTable();
+		
+		
+			dt.Columns.Add(new DataColumn("Id", typeof(string)));
+			dt.Columns.Add(new DataColumn("ContactName", typeof(string)));
+			dt.Columns.Add(new DataColumn("ContactTitle", typeof(string)));
+			dt.Columns.Add(new DataColumn("ContactAreaCode", typeof(string)));
+			dt.Columns.Add(new DataColumn("ContactPhone", typeof(string)));
+		
+			DataRow loRowToAdd;
+			loRowToAdd = dt.NewRow();
+			loRowToAdd[0] = "a";
+			loRowToAdd[1] = "b";
+			loRowToAdd[2] = "c";
+			loRowToAdd[3] = "d";
+			loRowToAdd[4] = "e";
+						
+			dt.Rows.Add(loRowToAdd);
+		
+			DataTable[] dtarr = new DataTable[] {dt}; 
+		
+			// Serialize the array elements.
+			formatter.Serialize(fs, dtarr);
+		
+			// Deserialize the array elements.
+			fs.Position = 0;
+			DataTable[] a2 = (DataTable[]) formatter.Deserialize(fs);
+		
+			DataSet ds = new DataSet();
+			ds.Tables.Add(a2[0]);
+		
+			StringWriter sw = new StringWriter ();
+			ds.WriteXml(sw);
+			XmlDocument doc = new XmlDocument ();
+			doc.LoadXml (sw.ToString ());
+			AssertEquals (5, doc.DocumentElement.FirstChild.ChildNodes.Count);
 		}
 	}
 }
