@@ -8,6 +8,27 @@
 // Copyright (C) Tim Coleman, 2002
 //
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System.CodeDom;
 using System.Web.Services;
 using System.Web.Services.Protocols;
@@ -91,20 +112,31 @@ namespace System.Web.Services.Description {
 			if (sab != null) location = sab.Location;
 			string url = GetServiceUrl (location); 
 
-			CodeTypeReference ctr = new CodeTypeReference ("System.Web.Services.Protocols.SoapHttpClientProtocol");
-			codeClass.BaseTypes.Add (ctr);
+			if (Style == ServiceDescriptionImportStyle.Client) {
+				CodeTypeReference ctr = new CodeTypeReference ("System.Web.Services.Protocols.SoapHttpClientProtocol");
+				codeClass.BaseTypes.Add (ctr);
+			}
+			else {
+				CodeTypeReference ctr = new CodeTypeReference ("System.Web.Services.WebService");
+				codeClass.BaseTypes.Add (ctr);
+				CodeAttributeDeclaration attws = new CodeAttributeDeclaration ("System.Web.Services.WebServiceAttribute");
+				attws.Arguments.Add (GetArg ("Namespace", Port.Binding.Namespace));
+				AddCustomAttribute (codeClass, attws, true);
+			}
 			
 			CodeAttributeDeclaration att = new CodeAttributeDeclaration ("System.Web.Services.WebServiceBinding");
 			att.Arguments.Add (GetArg ("Name", Port.Name));
 			att.Arguments.Add (GetArg ("Namespace", Port.Binding.Namespace));
 			AddCustomAttribute (codeClass, att, true);
 	
-			CodeConstructor cc = new CodeConstructor ();
-			cc.Attributes = MemberAttributes.Public;
-			CodeExpression ce = new CodeFieldReferenceExpression (new CodeThisReferenceExpression(), "Url");
-			CodeAssignStatement cas = new CodeAssignStatement (ce, new CodePrimitiveExpression (url));
-			cc.Statements.Add (cas);
-			codeClass.Members.Add (cc);
+			if (Style == ServiceDescriptionImportStyle.Client) {
+				CodeConstructor cc = new CodeConstructor ();
+				cc.Attributes = MemberAttributes.Public;
+				CodeExpression ce = new CodeFieldReferenceExpression (new CodeThisReferenceExpression(), "Url");
+				CodeAssignStatement cas = new CodeAssignStatement (ce, new CodePrimitiveExpression (url));
+				cc.Statements.Add (cas);
+				codeClass.Members.Add (cc);
+			}
 			
 			memberIds = new CodeIdentifiers ();
 			headerVariables = new Hashtable ();
@@ -113,10 +145,17 @@ namespace System.Web.Services.Description {
 		
 		protected override void BeginNamespace ()
 		{
+#if NET_2_0
+			xmlImporter = new XmlSchemaImporter (LiteralSchemas, base.CodeGenerationOptions, base.CodeGenerator, base.ImportContext);
+			soapImporter = new SoapSchemaImporter (EncodedSchemas, base.CodeGenerationOptions, base.CodeGenerator, base.ImportContext);
+			xmlExporter = new XmlCodeExporter (CodeNamespace, null, base.CodeGenerator, base.CodeGenerationOptions, null);
+			soapExporter = new SoapCodeExporter (CodeNamespace, null, base.CodeGenerator, base.CodeGenerationOptions, null);
+#else
 			xmlImporter = new XmlSchemaImporter (LiteralSchemas, ClassNames);
 			soapImporter = new SoapSchemaImporter (EncodedSchemas, ClassNames);
 			xmlExporter = new XmlCodeExporter (CodeNamespace, null);
 			soapExporter = new SoapCodeExporter (CodeNamespace, null);
+#endif
 		}
 
 		protected override void EndClass ()
@@ -150,6 +189,12 @@ namespace System.Web.Services.Description {
 			throw new NotImplementedException ();
 		}
 
+		[MonoTODO]
+		protected virtual bool IsSoapEncodingPresent (string uriList)
+		{
+			throw new NotImplementedException ();
+		}
+
 		protected override CodeMemberMethod GenerateMethod ()
 		{
 			try
@@ -212,7 +257,6 @@ namespace System.Web.Services.Description {
 		
 		XmlMembersMapping ImportMembersMapping (Message msg, SoapBodyBinding sbb, SoapBindingStyle style, bool output)
 		{
-			XmlQualifiedName elem = null;
 			string elemName = Operation.Name;
 			if (output) elemName += "Response";
 
@@ -274,9 +318,9 @@ namespace System.Web.Services.Description {
 			CodeMemberMethod method = new CodeMemberMethod ();
 			CodeMemberMethod methodBegin = new CodeMemberMethod ();
 			CodeMemberMethod methodEnd = new CodeMemberMethod ();
-			method.Attributes = MemberAttributes.Public;
-			methodBegin.Attributes = MemberAttributes.Public;
-			methodEnd.Attributes = MemberAttributes.Public;
+			method.Attributes = MemberAttributes.Public | MemberAttributes.Final;
+			methodBegin.Attributes = MemberAttributes.Public | MemberAttributes.Final;
+			methodEnd.Attributes = MemberAttributes.Public | MemberAttributes.Final;
 			
 			SoapBindingStyle style = soapOper.Style != SoapBindingStyle.Default ? soapOper.Style : soapBinding.Style;
 			
@@ -298,8 +342,8 @@ namespace System.Web.Services.Description {
 
 			method.Name = CodeIdentifier.MakeValid(Operation.Name);
 			if (method.Name == ClassName) method.Name += "1";
-			methodBegin.Name = memberIds.AddUnique(CodeIdentifier.MakeValid("Begin" + memberIds.MakeRightCase(method.Name)),method);
-			methodEnd.Name = memberIds.AddUnique(CodeIdentifier.MakeValid("End" + memberIds.MakeRightCase(method.Name)),method);
+			methodBegin.Name = memberIds.AddUnique(CodeIdentifier.MakeValid("Begin" + method.Name),method);
+			methodEnd.Name = memberIds.AddUnique(CodeIdentifier.MakeValid("End" + method.Name),method);
 
 			method.ReturnType = new CodeTypeReference (typeof(void));
 			methodEnd.ReturnType = new CodeTypeReference (typeof(void));
@@ -319,6 +363,7 @@ namespace System.Web.Services.Description {
 
 			if (outputMembers != null)
 			{
+				bool hasReturn = false;
 				for (int n=0; n<outputMembers.Count; n++)
 				{
 					CodeParameterDeclarationExpression cpd = GenerateParameter (outputMembers[n], FieldDirection.Out);
@@ -337,9 +382,9 @@ namespace System.Web.Services.Description {
 					
 					if (found) continue;
 	
-					if ((outputMembers [n].ElementName == Operation.Name + "Result") || 
-						(outputMembers.Count==1)) 
+					if (!hasReturn) 
 					{
+						hasReturn = true;
 						method.ReturnType = cpd.Type;
 						methodEnd.ReturnType = cpd.Type;
 						GenerateReturnAttributes (outputMembers, outputMembers[n], bodyBinding.Use, method);
@@ -385,42 +430,48 @@ namespace System.Web.Services.Description {
 				}
 			}
 			
-			// Invoke call
-			
-			CodeThisReferenceExpression ethis = new CodeThisReferenceExpression();
-			CodePrimitiveExpression varMsgName = new CodePrimitiveExpression (messageName);
-			CodeMethodInvokeExpression inv;
-			CodeVariableDeclarationStatement dec;
-
-			inv = new CodeMethodInvokeExpression (ethis, "Invoke", varMsgName, methodParams);
-			if (outputMembers != null && outputMembers.Count > 0)
+			if (Style == ServiceDescriptionImportStyle.Client) 
 			{
-				dec = new CodeVariableDeclarationStatement (typeof(object[]), varResults, inv);
-				method.Statements.Add (dec);
-				method.Statements.AddRange (outAssign);
+				// Invoke call
+				
+				CodeThisReferenceExpression ethis = new CodeThisReferenceExpression();
+				CodePrimitiveExpression varMsgName = new CodePrimitiveExpression (messageName);
+				CodeMethodInvokeExpression inv;
+				CodeVariableDeclarationStatement dec;
+	
+				inv = new CodeMethodInvokeExpression (ethis, "Invoke", varMsgName, methodParams);
+				if (outputMembers != null && outputMembers.Count > 0)
+				{
+					dec = new CodeVariableDeclarationStatement (typeof(object[]), varResults, inv);
+					method.Statements.Add (dec);
+					method.Statements.AddRange (outAssign);
+				}
+				else
+					method.Statements.Add (inv);
+				
+				// Begin Invoke Call
+				
+				CodeExpression expCallb = new CodeVariableReferenceExpression (varCallback);
+				CodeExpression expAsyncs = new CodeVariableReferenceExpression (varAsyncState);
+				inv = new CodeMethodInvokeExpression (ethis, "BeginInvoke", varMsgName, methodParams, expCallb, expAsyncs);
+				methodBegin.Statements.Add (new CodeMethodReturnStatement (inv));
+				
+				// End Invoke call
+				
+				CodeExpression varAsyncr = new CodeVariableReferenceExpression (varAsyncResult);
+				inv = new CodeMethodInvokeExpression (ethis, "EndInvoke", varAsyncr);
+				if (outputMembers != null && outputMembers.Count > 0)
+				{
+					dec = new CodeVariableDeclarationStatement (typeof(object[]), varResults, inv);
+					methodEnd.Statements.Add (dec);
+					methodEnd.Statements.AddRange (outAssign);
+				}
+				else
+					methodEnd.Statements.Add (inv);
 			}
-			else
-				method.Statements.Add (inv);
-			
-			// Begin Invoke Call
-			
-			CodeExpression expCallb = new CodeVariableReferenceExpression (varCallback);
-			CodeExpression expAsyncs = new CodeVariableReferenceExpression (varAsyncState);
-			inv = new CodeMethodInvokeExpression (ethis, "BeginInvoke", varMsgName, methodParams, expCallb, expAsyncs);
-			methodBegin.Statements.Add (new CodeMethodReturnStatement (inv));
-			
-			// End Invoke call
-			
-			CodeExpression varAsyncr = new CodeVariableReferenceExpression (varAsyncResult);
-			inv = new CodeMethodInvokeExpression (ethis, "EndInvoke", varAsyncr);
-			if (outputMembers != null && outputMembers.Count > 0)
-			{
-				dec = new CodeVariableDeclarationStatement (typeof(object[]), varResults, inv);
-				methodEnd.Statements.Add (dec);
-				methodEnd.Statements.AddRange (outAssign);
+			else {
+				method.Attributes = MemberAttributes.Public | MemberAttributes.Abstract;
 			}
-			else
-				methodEnd.Statements.Add (inv);
 			
 			// Attributes
 			
@@ -428,7 +479,7 @@ namespace System.Web.Services.Description {
 			
 			CodeAttributeDeclaration att = new CodeAttributeDeclaration ("System.Web.Services.WebMethodAttribute");
 			if (messageName != method.Name) att.Arguments.Add (GetArg ("MessageName",messageName));
-			AddCustomAttribute (method, att, false);
+			AddCustomAttribute (method, att, (Style == ServiceDescriptionImportStyle.Server));
 			
 			if (style == SoapBindingStyle.Rpc)
 			{
@@ -466,8 +517,11 @@ namespace System.Web.Services.Description {
 			AddCustomAttribute (method, att, true);
 			
 			CodeTypeDeclaration.Members.Add (method);
-			CodeTypeDeclaration.Members.Add (methodBegin);
-			CodeTypeDeclaration.Members.Add (methodEnd);
+			
+			if (Style == ServiceDescriptionImportStyle.Client) {
+				CodeTypeDeclaration.Members.Add (methodBegin);
+				CodeTypeDeclaration.Members.Add (methodEnd);
+			}
 			
 			return method;
 		}
@@ -555,7 +609,7 @@ namespace System.Web.Services.Description {
 			string varName = headerVariables [map] as string;
 			if (varName == null) 
 			{
-				varName = memberIds.AddUnique(CodeIdentifier.MakeValid (hb.Part + "Value"),hb);
+				varName = memberIds.AddUnique(CodeIdentifier.MakeValid (map.TypeName + "Value"),hb);
 				headerVariables.Add (map, varName);
 				CodeMemberField codeField = new CodeMemberField (map.TypeFullName, varName);
 				codeField.Attributes = MemberAttributes.Public;
