@@ -3,6 +3,7 @@ using System.IO;
 using System.Collections;
 using System.Text;
 using System.Xml;
+using System.Xml.Xsl;
 using System.Xml.Serialization;
 using System.ComponentModel;
 using System.Diagnostics;
@@ -42,7 +43,7 @@ public class Driver
 		
 		if (args[0] == "gp")
 		{
-			BuildProxies (GetArg (args,1) == "all");
+			BuildProxies (GetArg (args,1));
 		}
 		else if (args[0] == "gc")
 		{
@@ -64,6 +65,18 @@ public class Driver
 		{
 			Clean ();
 		}
+		else if (args[0] == "checkdiff")
+		{
+			CheckDiff (GetArg (args,1));
+		}
+		else if (args[0] == "msfix")
+		{
+			MsFix (GetArg (args,1));
+		}
+		else if (args[0] == "genfiles")
+		{
+			GenerateFiles (GetArg (args,1));
+		}
 		
 		SaveInfo ();
 		
@@ -469,27 +482,37 @@ public class Driver
 			Console.WriteLine ("{0,-3} {1}", serverCounts[n], serverNames[n]);
 	}
 
-	static void BuildProxies (bool buildAll)
+	static void BuildProxies (string host)
 	{
 		Console.WriteLine ();
 		Console.WriteLine ("Generating proxies");
 		Console.WriteLine ("------------------");
 		
+		XmlDocument doc = new XmlDocument ();
+		XmlElement ele = doc.CreateElement ("errors");
+		doc.AppendChild (ele);
+		
+		bool buildAll = host == "*";
+		if (buildAll) host = null;
+		
 		ArrayList proxies = new ArrayList ();
 		
-		foreach (ServiceData fd in services.services)
-			BuildProxy (fd, buildAll, proxies);
+		foreach (ServiceData fd in services.services) {
+			if (host != null && host != new Uri (fd.Wsdl).Host) continue;
+			BuildProxy (fd, buildAll, proxies, ele);
+		}
 		
 		StreamWriter sw = new StreamWriter (Path.Combine (basePath, "proxies.sources"));
 		foreach (string f in proxies)
 			sw.WriteLine (f);
 		sw.Close ();
 		
+		doc.Save ("proxy-gen-error.xml");		
 		Console.WriteLine ("Done");
 		Console.WriteLine ();
 	}
 	
-	static void BuildProxy (ServiceData fd, bool rebuild, ArrayList proxies)
+	static void BuildProxy (ServiceData fd, bool rebuild, ArrayList proxies, XmlElement errdoc)
 	{
 		string wsdl = GetWsdlFile (fd);
 		if (!File.Exists (wsdl)) 
@@ -547,16 +570,33 @@ public class Driver
 						File.Delete (pfile);
 				}
 				
-				ReportError ("Errors found while generating " + prot + " proxy for WSDL: " + wsdl, err);
+				WriteError (errdoc, ns, "Errors found while generating " + prot + " proxy for WSDL: " + wsdl, err);
 			}
 			else
 			{
-				Console.WriteLine ("OK");
-				proxies.Add (pfile);
+				if (File.Exists (pfile)) {
+					Console.WriteLine ("OK");
+					proxies.Add (pfile);
+				}
+				else {
+					Console.WriteLine ("FAIL");
+					string err = proc.StandardOutput.ReadToEnd ();
+					err += "\n" + proc.StandardError.ReadToEnd ();
+					WriteError (errdoc, ns, "Errors found while generating " + prot + " proxy for WSDL: " + wsdl, err);
+				}
 			}
 		}
 	}
 	
+	static void WriteError (XmlElement err, string ns, string error, string detail)
+	{
+		XmlElement ele = err.OwnerDocument.CreateElement ("error");
+		ele.SetAttribute ("ns",ns);
+		XmlText text = err.OwnerDocument.CreateTextNode (error + "\n" + detail);
+		ele.AppendChild (text);
+		err.AppendChild (ele);
+	}
+	
 	static void BuildClients (string wsdl)
 	{
 		StreamWriter sw = new StreamWriter (Path.Combine (basePath, "client.sources"));
@@ -609,6 +649,174 @@ public class Driver
 		Console.WriteLine ("Written file '" + file + "'");
 	}
 	
+	public static void CheckDiff (string diffFile)
+	{
+		int suc=0, tot=0, ign=0;
+		XmlDocument doc = new XmlDocument ();
+		doc.Load (diffFile);
+		
+		XmlDocument errdoc = new XmlDocument ();
+		errdoc.Load ("proxy-gen-error.xml");
+		
+		XmlDocument result = new XmlDocument ();
+		XmlElement res = result.CreateElement ("test-results");
+		res.SetAttribute ("name", "wsdl-tests");
+		res.SetAttribute ("date", DateTime.Now.ToShortDateString ());
+		res.SetAttribute ("time", DateTime.Now.ToShortTimeString ());
+		result.AppendChild (res);
+		
+		XmlElement ts = result.CreateElement ("test-suite");
+		ts.SetAttribute ("name", "wsdl");
+		ts.SetAttribute ("asserts", "0");
+		ts.SetAttribute ("time", "0");
+		res.AppendChild (ts);
+
+		XmlElement tsres = result.CreateElement ("results");
+		ts.AppendChild (tsres);
+		
+		XslTransform xsl = new XslTransform ();
+		xsl.Load ("cormissing.xsl");
+		
+		foreach (ServiceData sd in services.services)
+		{
+			if (sd.Protocols == null) continue;
+
+			foreach (string prot in sd.Protocols)
+			{
+				string ns = sd.Namespace + "." + prot;
+				
+				XmlElement elem = doc.SelectSingleNode ("assemblies/assembly/namespaces/namespace[@name='" + ns + "']") as XmlElement;
+				if (elem == null) {
+					ign++;
+					continue;
+				}
+				
+				if (!File.Exists (GetWsdlFile(sd))) {
+					Console.WriteLine ("WARNING: wsdl file not found: " + GetWsdlFile(sd));
+					ign++;
+					continue;
+				}
+				
+				tot++;
+
+				bool extrans = elem.GetAttribute ("presence") == "extra";
+				
+				if ((elem.GetAttribute ("error") != "" || elem.GetAttribute ("missing") != "" || elem.GetAttribute ("extra") != "") && !extrans)
+				{
+					XmlElement errelem = errdoc.SelectSingleNode ("/errors/error[@ns='" + ns + "']") as XmlElement;			
+					if (errelem != null) {
+						WriteResult (tsres, ns, false, sd.Wsdl + "\n" + errelem.InnerText);
+					}
+					else {
+						StringWriter str = new StringWriter ();
+						xsl.Transform (elem, null, str, null);
+						WriteResult (tsres, ns, false, sd.Wsdl + "\n" + str.ToString ());
+					}
+				}
+				else 
+				{
+					if (extrans) Console.WriteLine ("BONUS CLASS: " + ns);
+					suc++;
+					WriteResult (tsres, ns, true, sd.Wsdl);
+				}
+				
+			}
+		}
+		ts.SetAttribute ("success", (suc < tot).ToString());
+		
+		res.SetAttribute ("total", tot.ToString());
+		res.SetAttribute ("failures", (tot-suc).ToString());
+		res.SetAttribute ("not-run", "0");
+
+		result.Save ("WsdlTestResult.xml");
+		Console.WriteLine ("Compared:" + tot + " Ignored:" + ign + " Sucess:" + suc + " Fail:" + (tot - suc));
+	}
+	
+	static void WriteResult (XmlElement res, string name, bool success, string msg)
+	{
+		XmlDocument doc = res.OwnerDocument;
+		XmlElement test = doc.CreateElement ("test-case");
+		test.SetAttribute ("name", name);
+		test.SetAttribute ("executed", "True");
+		test.SetAttribute ("success", success.ToString());
+		test.SetAttribute ("time", "0");
+		test.SetAttribute ("asserts", "0");
+		
+		if (!success) {
+			XmlElement fail = doc.CreateElement ("failure");
+			test.AppendChild (fail);
+			XmlElement xmsg = doc.CreateElement ("message");
+			fail.AppendChild (xmsg);
+			xmsg.AppendChild (doc.CreateCDataSection (msg));
+			fail.AppendChild (doc.CreateElement ("stack-trace"));
+		}
+		res.AppendChild (test);
+	}
+	
+	static void MsFix (string fileName)
+	{
+		StreamReader sr = new StreamReader ("msfix");
+		XmlDocument doc = new XmlDocument ();
+		doc.Load (fileName);
+		
+		string line;
+		while ((line = sr.ReadLine ()) != null)
+		{
+			string[] ss = line.Split ('/');
+			XmlElement elem = doc.SelectSingleNode ("assemblies/assembly/namespaces/namespace[@name='" + ss[0] + "']/classes/class[@name='" + ss[1] + "']") as XmlElement;
+			if (elem != null) elem.ParentNode.RemoveChild (elem);
+		}
+		doc.Save (fileName);
+		sr.Close ();
+	}
+	
+	static void GenerateFiles (string templateFile)
+	{
+		XmlDocument doc = new XmlDocument ();
+		doc.Load (templateFile);
+		
+		XmlNodeList targets = doc.SelectNodes ("fileGeneration/targets/target");
+		XmlElement gvarset = (XmlElement) doc.SelectSingleNode ("fileGeneration/vars[@name='_global']");
+		
+		foreach (XmlElement node in targets) {
+			string tid = node.GetAttribute ("template");
+			string file = node.GetAttribute ("file");
+			XmlElement xtemp = (XmlElement) doc.SelectSingleNode ("fileGeneration/templates/template[@id='" + tid + "']");
+			if (xtemp == null) throw new Exception ("Template " + tid + " not found");
+			string tempFile = xtemp.GetAttribute ("file");
+			
+			StreamReader sr = new StreamReader (tempFile);
+			string template = sr.ReadToEnd ();
+			sr.Close ();
+			
+			string content = ReplaceVars (template, node);
+			content = ReplaceVars (content, gvarset);
+			
+			StreamWriter sw = new StreamWriter (file);
+			sw.Write (content);
+			sw.Close ();
+			Console.WriteLine ("Writen " + file);
+		}
+	}
+	
+	static string ReplaceVars (string content, XmlElement root)
+	{
+		XmlNodeList nodes = root.SelectNodes ("var");
+		foreach (XmlElement elem in nodes) {
+			string include = elem.GetAttribute ("include");
+			if (include != "") {
+				XmlElement varset = (XmlElement) root.OwnerDocument.SelectSingleNode ("fileGeneration/vars[@name='" + include + "']");
+				if (varset == null) throw new Exception ("varset '" + include + "' not found");
+				content = ReplaceVars (content, varset);
+			}
+			else {
+				content = content.Replace ("$" + elem.GetAttribute ("name") + "$", elem.InnerText);
+			}
+		}
+		
+		return content;
+	}
+	
 	static void RegisterFailure (ServiceData sd)
 	{
 		ServiceStatus status = null;
@@ -699,6 +907,11 @@ public class Driver
 		return Path.Combine (basePath, "wsdlcache");
 	}
 	
+	static string GetTempWsdlPath ()
+	{
+		return Path.Combine (basePath, "wsdlcache");
+	}
+	
 	static void CreateFolderForFile (string file)
 	{
 		string dir = Path.GetDirectoryName (file);
