@@ -6,6 +6,27 @@
 //
 // (C) 2003 Ximian, Inc (http://www.ximian.com)
 //
+
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
 using System;
 using System.CodeDom;
 using System.Collections;
@@ -695,9 +716,10 @@ namespace System.Web.Compilation
 				m.TargetObject = thisRef;
 				m.MethodName = builder.renderMethod.Name;
 
-				CodeObjectCreateExpression create = new CodeObjectCreateExpression ();
-				create.CreateType = new CodeTypeReference (typeof (RenderMethod));
-				create.Parameters.Add (m);
+				CodeDelegateCreateExpression create = new CodeDelegateCreateExpression ();
+				create.DelegateType = new CodeTypeReference (typeof (RenderMethod));
+				create.TargetObject = thisRef;
+				create.MethodName = builder.renderMethod.Name;
 
 				CodeMethodInvokeExpression invoke = new CodeMethodInvokeExpression ();
 				invoke.Method = new CodeMethodReferenceExpression (ctrlVar, "SetRenderMethodDelegate");
@@ -937,6 +959,9 @@ namespace System.Web.Compilation
 
 				if (c.IsKnownColor){
 					CodeFieldReferenceExpression expr = new CodeFieldReferenceExpression ();
+					if (c.IsSystemColor)
+						type = typeof (SystemColors);
+
 					expr.TargetObject = new CodeTypeReferenceExpression (type);
 					expr.FieldName = c.Name;
 					return expr;
@@ -953,6 +978,21 @@ namespace System.Web.Compilation
 				}
 			}
 
+			TypeConverter converter = TypeDescriptor.GetConverter (type);
+			if (converter != null && converter.CanConvertFrom (typeof (string))) {
+				CodeMethodReferenceExpression m = new CodeMethodReferenceExpression ();
+				m.TargetObject = new CodeTypeReferenceExpression (typeof (TypeDescriptor));
+				m.MethodName = "GetConverter";
+				CodeMethodInvokeExpression invoke = new CodeMethodInvokeExpression (m);
+				CodeTypeReference tref = new CodeTypeReference (type);
+				invoke.Parameters.Add (new CodeTypeOfExpression (tref));
+				
+				invoke = new CodeMethodInvokeExpression (invoke, "ConvertFrom");
+				invoke.Parameters.Add (new CodePrimitiveExpression (str));
+
+				return new CodeCastExpression (tref, invoke);
+			}
+
 			bool parms = false;
 			BindingFlags flags = BindingFlags.Public | BindingFlags.Static;
 			MethodInfo parse = type.GetMethod ("Parse", flags, null, arrayStringCultureInfo, null);
