@@ -7,6 +7,27 @@
 // (c) 2003,2004 Novell, Inc. (http://www.novell.com)
 //
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Configuration;
 using System.Collections;
@@ -140,6 +161,9 @@ namespace System.Web.Configuration
 				return (ConfigurationData) fileToConfig [WebConfigurationSettings.MachineConfigPath];
 
 			string dir = UrlUtils.GetDirectory (filepath);
+			if (HttpRuntime.AppDomainAppVirtualPath.Length > dir.Length)
+				return  (ConfigurationData) fileToConfig [WebConfigurationSettings.MachineConfigPath];
+
 			ConfigurationData data = (ConfigurationData) fileToConfig [dir];
 			if (data != null)
 				return data;
@@ -849,7 +873,8 @@ namespace System.Web.Configuration
 						ThrowException ("Unrecognized attribute in <configSections>.", reader);
 
 					MoveToNextElement (reader);
-					ReadSections (reader, null);
+					if (reader.Depth > depth)
+						ReadSections (reader, null);
 				} else if (name == "location") {
 					if (isLocation)
 						ThrowException ("<location> inside <location>", reader);
