@@ -9,6 +9,27 @@
 // (C) 2003 Andreas Nahr
 //
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections;
 using System.ComponentModel;
@@ -873,13 +894,17 @@ namespace System.Web.UI.WebControls
 					current.MergeStyle (rowStyle);
 					continue;
 				}
+				
+				if (current.HasControls ()) {
+					int len = current.Controls.Count;
+					for (int i = 0 ; i < len ; i++) {
+						Control currentCtrl = current.Controls [i];
+						if (!(currentCtrl is Table))
+							continue;
 
-				foreach (Control currentCtrl in current.Controls) {
-					if (!(currentCtrl is Table))
-						continue;
-
-					foreach (TableRow cRow in ((Table) currentCtrl).Rows)
-						cRow.MergeStyle (rowStyle);
+						foreach (TableRow cRow in ((Table) currentCtrl).Rows)
+							cRow.MergeStyle (rowStyle);
+					}
 				}
 			}
 		}
