@@ -10,6 +10,27 @@
 // (c) 2002,2003 Ximian, Inc. (http://www.ximian.com)
 //
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections;
 using System.ComponentModel;
@@ -26,31 +47,33 @@ namespace System.Web.UI
 	[DesignerSerializer ("Microsoft.VSDesigner.WebForms.ControlCodeDomSerializer, " + Consts.AssemblyMicrosoft_VSDesigner, "System.ComponentModel.Design.Serialization.CodeDomSerializer, " + Consts.AssemblySystem_Design)]
         public class Control : IComponent, IDisposable, IParserAccessor, IDataBindingsAccessor
         {
-                private static readonly object DataBindingEvent = new object();
-                private static readonly object DisposedEvent = new object();
-                private static readonly object InitEvent = new object();
-                private static readonly object LoadEvent = new object();
-                private static readonly object PreRenderEvent = new object();
-                private static readonly object UnloadEvent = new object();
+		private static readonly object DataBindingEvent = new object();
+		private static readonly object DisposedEvent = new object();
+		private static readonly object InitEvent = new object();
+		private static readonly object LoadEvent = new object();
+		private static readonly object PreRenderEvent = new object();
+		private static readonly object UnloadEvent = new object();
+		private static string[] defaultNameArray;
+		private string _templateSourceDir;
 		private string uniqueID;
-                private string _userId;
+		private string _userId;
 		private bool id_set;
-                private ControlCollection _controls;
-                private bool _enableViewState = true;
-                private IDictionary _childViewStates;
-                private bool _isNamingContainer;
-                private Control _namingContainer;
-                private Page _page;
-                private Control _parent;
-                private ISite _site;
-                private bool _visible = true;
-                private bool visibleChanged;
-                private HttpContext _context;
-                private bool _childControlsCreated;
-                private StateBag _viewState;
-                private bool _trackViewState;
-                private EventHandlerList _events = new EventHandlerList();
-                private RenderMethod _renderMethodDelegate;
+		private ControlCollection _controls;
+		private bool _enableViewState = true;
+		private IDictionary _childViewStates;
+		private bool _isNamingContainer;
+		private Control _namingContainer;
+		private Page _page;
+		private Control _parent;
+		private ISite _site;
+		private bool _visible = true;
+		private bool visibleChanged;
+		private HttpContext _context;
+		private bool _childControlsCreated;
+		private StateBag _viewState;
+		private bool _trackViewState;
+		private EventHandlerList _events = new EventHandlerList();
+		private RenderMethod _renderMethodDelegate;
 		private bool autoID = true;
 		private bool creatingControls;
 		private bool bindingContainer = true;
@@ -64,6 +87,13 @@ namespace System.Web.UI
  
 		DataBindingCollection dataBindings;
 		Hashtable pendingVS; // may hold unused viewstate data from child controls
+		
+		static Control ()
+		{
+			defaultNameArray = new string [100];
+			for (int i = 0 ; i < 100 ; i++)
+				defaultNameArray [i] = "_ctrl" + i;
+		}
 
                 public Control()
                 {
@@ -203,7 +233,7 @@ namespace System.Web.UI
 		[Browsable (false)]
 		[WebSysDescription ("A virtual directory containing the parent of the control.")]
                 public virtual string TemplateSourceDirectory {
-                        get { return (_parent == null) ? String.Empty : _parent.TemplateSourceDirectory; }
+			get { return (_parent == null) ? String.Empty : _parent.TemplateSourceDirectory; }
                 }
 
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
@@ -264,10 +294,13 @@ namespace System.Web.UI
                         set
                         {
                                 if (value == false && _childControlsCreated == true)
-                                        _controls.Clear();
+                                        Controls.Clear();
                                 _childControlsCreated = value;
                         }
                 }
+
+		[Browsable (false)]
+		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
                 protected virtual HttpContext Context //DIT
                 {
                         get
@@ -309,6 +342,10 @@ namespace System.Web.UI
                                 return _trackViewState;
                         }
                 }
+
+		[Browsable (false)]
+		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
+		[WebSysDescription ("ViewState")]
                 protected virtual StateBag ViewState
                 {
                         get
@@ -323,6 +360,8 @@ namespace System.Web.UI
                         }
                 }
 
+		[Browsable (false)]
+		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
                 protected virtual bool ViewStateIgnoresCase
                 {
                         get {
@@ -344,22 +383,28 @@ namespace System.Web.UI
 		{
 			defaultNumberID = 0;
 		}
-		
+
 		string GetDefaultName ()
 		{
-			return "_ctrl" + defaultNumberID++;
+			string defaultName;
+			if (defaultNumberID > 99) {
+				defaultName = "_ctrl" + defaultNumberID++;
+			} else {
+				defaultName = defaultNameArray [defaultNumberID++];
+			}
+			return defaultName;
 		}
-		
+
 		void NullifyUniqueID ()
 		{
 			uniqueID = null;
-			if (_controls == null)
+			if (!HasControls ())
 				return;
 
-			foreach (Control c in _controls)
+			foreach (Control c in Controls)
 				c.NullifyUniqueID ();
 		}
-		
+
 		protected internal virtual void AddedControl (Control control, int index)
 		{
 			/* Ensure the control don't have more than 1 parent */
@@ -437,10 +482,9 @@ namespace System.Web.UI
 
 		protected bool IsLiteralContent()
 		{
-			if (_controls != null) 
-				if (_controls.Count == 1)
-					if (_controls[0] is LiteralControl)
-						return true;
+			if (HasControls () && Controls.Count == 1 && (Controls [0] is LiteralControl))
+				return true;
+
 			return false;
 		}
 
@@ -451,14 +495,14 @@ namespace System.Web.UI
 
 		Control LookForControlByName (string id)
 		{
-			if (!HasChildren)
+			if (!HasControls ())
 				return null;
 
-			foreach (Control c in _controls) {
+			foreach (Control c in Controls) {
 				if (String.Compare (id, c._userId, true) == 0)
 					return c;
 
-				if (!c._isNamingContainer && c.HasChildren) {
+				if (!c._isNamingContainer && c.HasControls ()) {
 					Control child = c.LookForControlByName (id);
 					if (child != null)
 						return child;
@@ -471,7 +515,7 @@ namespace System.Web.UI
                 protected virtual Control FindControl (string id, int pathOffset)
                 {
 			EnsureChildControls ();
-			if (_controls == null)
+			if (!HasControls ())
 				return null;
 
 			Control namingContainer = null;
@@ -574,13 +618,17 @@ namespace System.Web.UI
                         RenderChildren(writer);
                 }
 
-                protected virtual void RenderChildren(HtmlTextWriter writer) //DIT
+                protected virtual void RenderChildren (HtmlTextWriter writer) //DIT
                 {
-                        if (_renderMethodDelegate != null)
-                                _renderMethodDelegate(writer, this);
-                        else if (_controls != null)
-                                foreach (Control c in _controls)
-                                        c.RenderControl(writer);
+                        if (_renderMethodDelegate != null) {
+                                _renderMethodDelegate (writer, this);
+			} else if (HasControls ()) {
+				int len = Controls.Count;
+				for (int i = 0; i < len; i++) {
+					Control c = Controls [i];
+					c.RenderControl (writer);
+				}
+			}
                 }
 
                 protected virtual object SaveViewState ()
@@ -611,11 +659,6 @@ namespace System.Web.UI
                         }
                 }
 
-		internal bool HasChildren
-		{
-			get { return (_controls != null && _controls.Count > 0); }
-		}
-
 		[WebCategory ("FIXME")]
 		[WebSysDescription ("Raised when the contols databound properties are evaluated.")]
                 public event EventHandler DataBinding //DIT
@@ -726,11 +769,14 @@ namespace System.Web.UI
 		
 		void DataBindChildren ()
 		{
-			if (_controls == null)
+			if (!HasControls ())
 				return;
 			
-			foreach (Control c in _controls)
-				c.DataBind();
+			int len = Controls.Count;
+			for (int i = 0; i < len; i++) {
+				Control c = Controls [i];
+				c.DataBind ();
+			}
 		}
 
 
@@ -757,7 +803,10 @@ namespace System.Web.UI
 				return relativeUrl;
 			
 			string ts = TemplateSourceDirectory;
-			if (ts == "" || UrlUtils.IsRelativeUrl (relativeUrl) == false)
+			if (ts == "" || !UrlUtils.IsRelativeUrl (relativeUrl))
+				return relativeUrl;
+
+			if (relativeUrl.IndexOf ('/') == -1 && relativeUrl [0] != '.' && relativeUrl != "..")
 				return relativeUrl;
 
 			HttpResponse resp = Context.Response;
@@ -773,18 +822,26 @@ namespace System.Web.UI
                 internal void LoadRecursive()
                 {
                         OnLoad (EventArgs.Empty);
-                        if (_controls != null) {
-				foreach (Control c in _controls)
+                        if (HasControls ()) {
+				int len = Controls.Count;
+				for (int i=0;i<len;i++)
+				{
+					Control c = Controls[i];
 					c.LoadRecursive ();
+				}
 			}
 			loaded = true;
                 }
 
                 internal void UnloadRecursive(Boolean dispose)
                 {
-                        if (_controls != null) {
-				foreach (Control c in _controls)
+			if (HasControls ()) {
+				int len = Controls.Count;
+				for (int i=0;i<len;i++)
+				{
+					Control c = Controls[i];					
 					c.UnloadRecursive (dispose);
+				}
 			}
 
                         OnUnload (EventArgs.Empty);
@@ -797,18 +854,22 @@ namespace System.Web.UI
 			if (_visible) {
 				EnsureChildControls ();
 				OnPreRender (EventArgs.Empty);
-				if (_controls == null)
+				if (!HasControls ())
 					return;
-
-				foreach (Control c in _controls)
+				
+				int len = Controls.Count;
+				for (int i=0;i<len;i++)
+				{
+					Control c = Controls[i];
 					c.PreRenderRecursiveInternal ();
+				}
 			}
 			prerendered = true;
                 }
 
                 internal void InitRecursive(Control namingContainer)
                 {
-                        if (_controls != null) {
+                        if (HasControls ()) {
 				if (_isNamingContainer)
 					namingContainer = this;
 
@@ -817,13 +878,15 @@ namespace System.Web.UI
 				    namingContainer.autoID)
 					namingContainer._userId = namingContainer.GetDefaultName () + "b";
 
-				foreach (Control c in _controls) {
+				int len = Controls.Count;
+				for (int i=0;i<len;i++)
+				{
+					Control c = Controls[i];
 					c._page = Page;
 					c._namingContainer = namingContainer;
 					if (namingContainer != null && c._userId == null && c.autoID)
 						c._userId = namingContainer.GetDefaultName () + "c";
-
-					c.InitRecursive (namingContainer);
+					c.InitRecursive (namingContainer);	
 				}
 			}
 
@@ -843,19 +906,26 @@ namespace System.Web.UI
 			ArrayList controlStates = null;
 
 			int idx = -1;
-			foreach (Control ctrl in Controls) {
-				object ctrlState = ctrl.SaveViewStateRecursive ();
-				idx++;
-				if (ctrlState == null)
-					continue;
+			if (HasControls ())
+			{
+				int len = Controls.Count;
+				for (int i=0;i<len;i++)
+				{
+					Control ctrl = Controls[i];
+					object ctrlState = ctrl.SaveViewStateRecursive ();
+					idx++;
+					if (ctrlState == null)
+						continue;
 
-				if (controlList == null) {
-					controlList = new ArrayList ();
-					controlStates = new ArrayList ();
+					if (controlList == null) 
+					{
+						controlList = new ArrayList ();
+						controlStates = new ArrayList ();
+					}
+
+					controlList.Add (idx);
+					controlStates.Add (ctrlState);
 				}
-
-				controlList.Add (idx);
-				controlStates.Add (ctrlState);
 			}
 
 			object thisState = SaveViewState ();
