@@ -1,3 +1,24 @@
+
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
 /*	System.Web.UI
 *	Authors
 *		Leen Toelen (toelen@hotmail.com)
@@ -14,7 +35,8 @@ namespace System.Web.UI{
 public class HtmlTextWriter : System.IO.TextWriter {
 
 static HtmlTextWriter(){
-	HtmlTextWriter._tagKeyLookupTable = new Hashtable(97);
+	HtmlTextWriter._tagKeyLookupTable = new Hashtable(97,CaseInsensitiveHashCodeProvider.Default,
+							  CaseInsensitiveComparer.Default);
 	HtmlTextWriter._tagNameLookupArray = new TagInformation[97];
 	HtmlTextWriter.RegisterTag("", HtmlTextWriterTag.Unknown, TagType.Other);
 	HtmlTextWriter.RegisterTag("a", HtmlTextWriterTag.A, TagType.Inline);
@@ -114,7 +136,8 @@ static HtmlTextWriter(){
 	HtmlTextWriter.RegisterTag("wbr", HtmlTextWriterTag.Wbr, TagType.NonClosing);
 	HtmlTextWriter.RegisterTag("xml", HtmlTextWriterTag.Xml, TagType.Other);
 
-	HtmlTextWriter._attrKeyLookupTable = new Hashtable(40);
+	HtmlTextWriter._attrKeyLookupTable = new Hashtable(40,CaseInsensitiveHashCodeProvider.Default,
+							   CaseInsensitiveComparer.Default);
 	HtmlTextWriter._attrNameLookupArray = new AttributeInformation[40];
 	HtmlTextWriter.RegisterAttribute("accesskey", HtmlTextWriterAttribute.Accesskey, true);
 	HtmlTextWriter.RegisterAttribute("align", HtmlTextWriterAttribute.Align, false);
@@ -157,7 +180,8 @@ static HtmlTextWriter(){
 	HtmlTextWriter.RegisterAttribute("width", HtmlTextWriterAttribute.Width, false);
 	HtmlTextWriter.RegisterAttribute("wrap", HtmlTextWriterAttribute.Wrap, false);
 
-	HtmlTextWriter._styleKeyLookupTable = new Hashtable(14);
+	HtmlTextWriter._styleKeyLookupTable = new Hashtable(14,CaseInsensitiveHashCodeProvider.Default,
+								CaseInsensitiveComparer.Default);
 	HtmlTextWriter._styleNameLookupArray = new String[14];
 	HtmlTextWriter.RegisterStyle("background-color", HtmlTextWriterStyle.BackgroundColor);
 	HtmlTextWriter.RegisterStyle("background-image", HtmlTextWriterStyle.BackgroundImage);
@@ -314,7 +338,7 @@ public override void Flush(){
 
 protected HtmlTextWriterAttribute GetAttributeKey(string attrName){
 	if (attrName != null && attrName.Length > 0) {
-		object attr = HtmlTextWriter._attrKeyLookupTable[attrName.ToLower()];
+		object attr = HtmlTextWriter._attrKeyLookupTable[attrName];
 		if (attr != null)
 			return (HtmlTextWriterAttribute) attr;
 	}
@@ -329,7 +353,7 @@ protected string GetAttributeName(HtmlTextWriterAttribute attrKey){
 
 protected HtmlTextWriterStyle GetStyleKey(string styleName){
 	if (styleName != null && styleName.Length > 0) {
-		object style = HtmlTextWriter._styleKeyLookupTable[styleName.ToLower()];
+		object style = HtmlTextWriter._styleKeyLookupTable[styleName];
 		if (style != null)
 			return (HtmlTextWriterStyle) style;
 	}
@@ -344,7 +368,7 @@ protected string GetStyleName(HtmlTextWriterStyle styleKey){
 
 protected virtual HtmlTextWriterTag GetTagKey(string tagName){
 	if (tagName != null && tagName.Length > 0) {
-		object tag = HtmlTextWriter._tagKeyLookupTable[tagName.ToLower()];
+		object tag = HtmlTextWriter._tagKeyLookupTable[tagName];
 		if (tag != null)
 			return (HtmlTextWriterTag) tag;
 	}
@@ -365,7 +389,7 @@ protected bool IsAttributeDefined(HtmlTextWriterAttribute key){
 	return false;
 }
 
-protected bool IsAttributeDefined(HtmlTextWriterAttribute key, ref string value){
+protected bool IsAttributeDefined(HtmlTextWriterAttribute key, out string value){
 	value = null;
 	for (int i=0; i < _attrCount; i++) {
 		if (_attrList[i].key == key) {
@@ -384,7 +408,7 @@ protected bool IsStyleAttributeDefined(HtmlTextWriterStyle key){
 	return false;
 }
 
-protected bool IsStyleAttributeDefined(HtmlTextWriterStyle key, ref string value){
+protected bool IsStyleAttributeDefined(HtmlTextWriterStyle key, out string value){
 	value = null;
 	for( int i=0; i < _styleCount; i++) {
 		if (_styleList[i].key == key) {
@@ -499,6 +523,10 @@ public virtual void RenderBeginTag(HtmlTextWriterTag tagKey){
 		}
 	}
 	TagInformation currentTag = HtmlTextWriter._tagNameLookupArray[_tagIndex];
+	if (currentTag.closingTag == null && currentTag.tagType == TagType.Other) {
+		currentTag.closingTag = EndTagLeftChars + _tagName + TagRightChar;
+	}
+
 	if (tagRender) {
 		tagRendered = false;
 		if (tabsPending)
