@@ -10,6 +10,27 @@
 // (c) 2003 Novell, Inc. (http://www.novell.com)
 //
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections;
 using System.Collections.Specialized;
@@ -29,7 +50,6 @@ using System.Web.Util;
 namespace System.Web.UI
 {
 
-[MonoTODO ("FIXME missing the IRootDesigner Attribute")]
 [DefaultEvent ("Load"), DesignerCategory ("ASPXCodeBehind")]
 [ToolboxItem (false)]
 [Designer ("System.Web.UI.Design.ControlDesigner, " + Consts.AssemblySystem_Design, typeof (IDesigner))]
@@ -61,6 +81,7 @@ public class Page : TemplateControl, IHttpHandler
 	bool handleViewState;
 	string viewStateUserKey;
 	NameValueCollection _requestValueCollection;
+	string clientTarget;
 
 	[EditorBrowsable (EditorBrowsableState.Never)]
 	protected const string postEventArgumentID = "__EVENTARGUMENT";
@@ -103,14 +124,17 @@ public class Page : TemplateControl, IHttpHandler
 		get { return _context.Cache; }
 	}
 
-	[MonoTODO]
 	[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 	[Browsable (false), DefaultValue ("")]
 	[WebSysDescription ("Value do override the automatic browser detection and force the page to use the specified browser.")]
 	public string ClientTarget
 	{
-		get { throw new NotImplementedException (); }
-		set { throw new NotImplementedException (); }
+		get { return (clientTarget == null) ? "" : clientTarget; }
+		set {
+			clientTarget = value;
+			if (value == "")
+				clientTarget = null;
+		}
 	}
 
 	[EditorBrowsable (EditorBrowsableState.Never)]
@@ -189,7 +213,7 @@ public class Page : TemplateControl, IHttpHandler
 	public bool IsPostBack
 	{
 		get {
-			return (0 == String.Compare (Request.HttpMethod, "POST", true));
+			return _requestValueCollection != null;
 		}
 	}
 
@@ -364,7 +388,7 @@ public class Page : TemplateControl, IHttpHandler
 			return null;
 
 		NameValueCollection coll = null;
-		if (IsPostBack)
+		if (0 == String.Compare (Request.HttpMethod, "POST", true))
 			coll =	req.Form;
 		else 
 			coll = req.QueryString;
@@ -502,7 +526,12 @@ public class Page : TemplateControl, IHttpHandler
 		writer.WriteLine ("<script language=\"javascript\">");
 		writer.WriteLine ("<!--");
 		writer.WriteLine ("\tfunction __doPostBack(eventTarget, eventArgument) {");
-		writer.WriteLine ("\t\tvar theform = document.getElementById ('{0}');", formUniqueID);
+
+		if (Request.Browser.Browser == ("Netscape") && Request.Browser.MajorVersion == 4)
+			writer.WriteLine ("\t\tvar theform = document.{0};", formUniqueID);
+		else
+			writer.WriteLine ("\t\tvar theform = document.getElementById ('{0}');", formUniqueID);
+
 		writer.WriteLine ("\t\ttheform.{0}.value = eventTarget;", postEventSourceID);
 		writer.WriteLine ("\t\ttheform.{0}.value = eventArgument;", postEventArgumentID);
 		writer.WriteLine ("\t\ttheform.submit();");
@@ -547,8 +576,9 @@ public class Page : TemplateControl, IHttpHandler
 		}
 
 		if (handleViewState) {
+			string vs = GetViewStateString ();
 			writer.Write ("<input type=\"hidden\" name=\"__VIEWSTATE\" ");
-			writer.WriteLine ("value=\"{0}\" />", GetViewStateString ());
+			writer.WriteLine ("value=\"{0}\" />", vs);
 		}
 
 		WriteScripts (writer, clientScriptBlocks);
@@ -556,6 +586,8 @@ public class Page : TemplateControl, IHttpHandler
 
 	internal string GetViewStateString ()
 	{
+		if (_savedViewState == null)
+			return null;
 		StringWriter sr = new StringWriter ();
 		LosFormatter fmt = new LosFormatter ();
 		fmt.Serialize (sr, _savedViewState);
@@ -646,21 +678,26 @@ public class Page : TemplateControl, IHttpHandler
 			}
 		}
 
+		ArrayList list1 = null;
 		if (_requiresPostBackCopy != null && _requiresPostBackCopy.Count > 0) {
 			string [] handlers = (string []) _requiresPostBackCopy.ToArray (typeof (string));
 			foreach (string id in handlers) {
 				IPostBackDataHandler pbdh = FindControl (id) as IPostBackDataHandler;
-				if (pbdh == null)
-					continue;
-			
-				_requiresPostBackCopy.Remove (id);
-				if (pbdh.LoadPostData (id, data)) {
-					if (requiresPostDataChanged == null)
-						requiresPostDataChanged = new ArrayList ();
-
-					requiresPostDataChanged.Add (pbdh);
+				if (pbdh != null) {			
+					_requiresPostBackCopy.Remove (id);
+					if (pbdh.LoadPostData (id, data)) {
+						if (requiresPostDataChanged == null)
+							requiresPostDataChanged = new ArrayList ();
+	
+						requiresPostDataChanged.Add (pbdh);
+					}
+				} else if (second) {
+					if (list1 == null)
+						list1 = new ArrayList ();
+					list1.Add (id);
 				}
 			}
+			_requiresPostBack = list1;
 		}
 	}
 
@@ -668,6 +705,9 @@ public class Page : TemplateControl, IHttpHandler
 	public void ProcessRequest (HttpContext context)
 	{
 		_context = context;
+		if (clientTarget != null)
+			Request.ClientTarget = clientTarget;
+
 		WireupAutomaticEvents ();
 		//-- Control execution lifecycle in the docs
 
@@ -732,19 +772,21 @@ public class Page : TemplateControl, IHttpHandler
 		Trace.Write ("aspx.page", "End Render");
 		
 		RenderTrace (output);
-		_context = null;
 	}
 
 	private void RenderTrace (HtmlTextWriter output)
 	{
-		TraceManager manager = HttpRuntime.TraceManager;
-		
-		if (!Trace.IsEnabled && !manager.Enabled)
+		TraceManager traceManager = HttpRuntime.TraceManager;
+
+		if (Trace.HaveTrace && !Trace.IsEnabled || !Trace.HaveTrace && !traceManager.Enabled)
 			return;
 		
 		Trace.SaveData ();
-		
-		if (Trace.IsEnabled || manager.PageOutput)
+
+		if (!Trace.HaveTrace && traceManager.Enabled && !traceManager.PageOutput) 
+			return;
+
+		if (!traceManager.LocalOnly || Context.Request.IsLocal)
 			Trace.Render (output);
 	}
 	
@@ -830,8 +872,6 @@ public class Page : TemplateControl, IHttpHandler
 		throw new NotImplementedException ();
 	}
 
-
-	[MonoTODO]
 	[EditorBrowsable (EditorBrowsableState.Advanced)]
 	public void RegisterOnSubmitStatement (string key, string script)
 	{
