@@ -9,6 +9,27 @@
 // (c) Copyright 2002-2003 Ximian, Inc. (http://www.ximian.com)
 // (c) Copyright 2004 Novell, Inc. (http://www.novell.com)
 //
+
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
 using System;
 using System.Collections;
 using System.ComponentModel;
@@ -784,7 +805,12 @@ namespace System.Web
 					return ret;
 				} catch (DirectoryNotFoundException) {
 					throw new HttpException (404, "Cannot find '" + file + "'.");
-				} catch (FileNotFoundException) {
+				} catch (FileNotFoundException fnf) {
+					string fname = fnf.FileName;
+					if (fname != null && fname != "") {
+						file = Path.GetFileName (fname);
+					}
+
 					throw new HttpException (404, "Cannot find '" + file + "'.");
 				}
 			}
@@ -794,11 +820,13 @@ namespace System.Web
 
 		internal void RecycleHandlers ()
 		{
-			if (null == _recycleHandlers)
+			if (null == _recycleHandlers || _recycleHandlers.Count == 0)
 				return;
 
 			foreach (HandlerFactory item in _recycleHandlers) 
 				item.Factory.ReleaseHandler (item.Handler);
+				
+			_recycleHandlers.Clear ();
 		}
 
 		internal void InitModules ()
@@ -866,9 +894,7 @@ namespace System.Web
 			_savedContext = HttpContext.Context;
 			HttpContext.Context = _Context;
 			HttpRuntime.TimeoutManager.Add (_Context);
-
-			_savedUser = Thread.CurrentPrincipal;
-			Thread.CurrentPrincipal = Context.User;	
+			SetPrincipal (Context.User);
 		}
 
 		internal void OnStateExecuteLeave ()
@@ -876,10 +902,25 @@ namespace System.Web
 			RestoreThreadCulture ();
 			HttpRuntime.TimeoutManager.Remove (_Context);
 			HttpContext.Context = _savedContext;
-			if (null != _savedUser)  {
-				Thread.CurrentPrincipal = _savedUser;
-				_savedUser = null;
-			}
+			RestorePrincipal ();
+		}
+
+		internal void SetPrincipal (IPrincipal principal)
+		{
+			// Don't overwrite _savedUser if called from inside a step
+			if (_savedUser == null)
+				_savedUser = Thread.CurrentPrincipal;
+
+			Thread.CurrentPrincipal = principal;
+		}
+
+		internal void RestorePrincipal ()
+		{
+			if (_savedUser == null)
+				return;
+
+			Thread.CurrentPrincipal = _savedUser;
+			_savedUser = null;
 		}
 
 		internal void ClearError ()
@@ -1109,10 +1150,8 @@ namespace System.Web
 				if (null != _Session)
 					return _Session;
 
-				if (null != _Context && null != _Context.Session) {
-					_Session = _Context.Session;
-					return _Session;
-				}
+				if (null != _Context && null != _Context.Session)
+					return _Context.Session;
 
 				throw new HttpException ("Failed to get session object");
 			}
